%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of forecasts, %
%          forecast uncertainty (including parameter uncertainty)%
%          and the actual outcomes. Forecasts may be either      %
%          within-sample or out-of-sample. In the latter case    %
%          we re-estimate all parameters using the smaller       %
%          sample                                                %
%                                                                %
%          Function: forecastdlg                                 %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       c_mod: structure with model info         %
%                       restrict: 1x2 indicator vector if        %
%                          restricted estimates of alpha and     %
%                          beta are used                         %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        May 4, 2006                      %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = forecastdlg(selector,c_mod,restrict)

if strcmp(selector,'init')==1;
   tm = c_mod;
   tm.restrict = restrict;
   %
   % setting up the actual sample period
   %
   [tm.dataindex,tm.samplebeg,tm.sampleend] = make_dataindex(tm.freq,tm.samplebeg,tm.sampleend);
   checkyear = floor(tm.samplebeg);
   checkperiod = tm.samplebeg-checkyear;
   tm.subbeginyear = num2str(checkyear);
   if strcmp(tm.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      tm.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(tm.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      tm.subbeginperiod = num2str(round(checkperiod));
   end;
   %
   % create sample strings
   %
   tm.sample_string = '';
   for i=tm.df+4:tm.T;
      tm.sample_string = strvcat(tm.sample_string,getdate(tm.samplebeg,tm.sampleend,tm.freq,i/tm.T));
   end;
   %
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
else;
   thisexit = 1;
   forecastgui = findobj('Type','figure','Tag','ForecastGUI')';
   f_cast = get(forecastgui,'UserData');
   curr_ini = get(f_cast.popup,'UserData');
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      forecastgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','forecastdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-360)/2 (realscrsz(4)-210)/2 360 210], ...
         'Tag','ForecastGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Forecasts', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','forecastdlg wayout', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(forecastgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 358 200],'Forecast Selections',96,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[306 135 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.forecastbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      %
      % Forecast period selection
      %

      f_cast.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 166 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast period:', ...
         'HorizontalAlignment','left', ...
         'Tag','PeriodText');

      f_cast.begin = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 148 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.sample_string, ...
         'Callback','forecastdlg checkperiod', ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',max(1,size(tm.sample_string,1)-tm.h+1));

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[116 150 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','_', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.end = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[126 148 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.sample_string, ...
         'Callback','forecastdlg checkperiod', ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',size(tm.sample_string,1));

      %
      % Select forecast type
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 113 100 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast format:', ...
         'HorizontalAlignment','left', ...
         'Tag','FormatText');


      f_cast.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 95 210 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','Levels|Growth rates|Levels & growth rates', ...
         'HorizontalAlignment','center', ...
         'Tag','TypePopup',...
         'Value',1);

      %
      % Forecast method
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 60 100 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast method:', ...
         'HorizontalAlignment','left', ...
         'Tag','MethodText');

      f_cast.method1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[16 40 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','radiobutton', ...
         'Value',1, ...
         'Enable','on', ...
         'Visible','on', ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','forecastdlg method', ...
      	 'String','Within-sample forecasts', ...
      	 'Tag','WithinSampleButton');

      f_cast.method2 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[16 20 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','radiobutton', ...
         'Value',0, ...
         'Enable','on', ...
         'Visible','on', ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','forecastdlg method', ...
      	 'String','Out-of-sample forecasts', ...
      	 'Tag','OutOfSampleButton');

      if curr_ini.outofsample==1;
         set(f_cast.method1,'Value',0);
         set(f_cast.method2,'Value',1);
      end;

      %
      % Create graph button
      %

      f_cast.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[252 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','forecastdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','ShowButton');

      %
      %  Done button
      %

      f_cast.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[252 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','forecastdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(forecastgui,'UserData',f_cast,...
         'HandleVisibility','callback');

   case 'showgraph'
      %
      % Determine the sample
      %
      f_sample_begin = get(f_cast.begin,'Value')+(tm.df+3);
      f_sample_end = get(f_cast.end,'Value')+(tm.df+3);
      est_sample_end = get(f_cast.begin,'Value')+(tm.df+2);
      d_local = tm.d;
      %
      % check method
      %
      if get(f_cast.method2,'Value')==1;
         type_str = 'Out-of-sample';
         tag_str = 'oos-';
         %
         % need to check if any deterministic variables should be deleted (may need to change locally tm.d)
         %
         if isempty(tm.x1)==0;
            x1 = tm.x1(1:tm.p+est_sample_end,:);
         else;
            x1 = [];
         end;
         if isempty(tm.x1_r)==0;
            x1_r = tm.x1_r(1:tm.p+est_sample_end,:);
         else;
            x1_r = [];
         end;
         x0 = tm.x0(1:tm.p+est_sample_end,:);
         [x1,deter,deterdel] = checkdeter(x1,x1_r,tm.deter);
         d_local = size(x1,2);
         [Z0,Z1,Z2,Z1t] = vecmdata(x0,tm.p,x1);
         %
         if isempty(x1_r)==0;
            num = size(x1_r,1);
            x1_r = x1_r(tm.p+1:num,:);
            Z1 = [Z1;x1_r'];
         end;
         %
         % we need to re-estimate everything
         %
         T = size(Z0,2);
         %
         % Display a dialog to let the user know what is going on unless
         % we're estimating the minimum
         %
         if (tm.restrict(1,2)==1)|((isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0));
            txt = strvcat(' ','Re-estimating parameters.','Please hold!');
            mywait = waitdlg(txt,'information');
            waitf = get(mywait,'UserData');
         end;
         %
         [S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,trace_test] = mlcointbasic(Z0,Z1,Z2,tm.n);
         if tm.r>0;
            beta = beta(:,1:tm.r);
         else;
            beta = [];
         end;
         if tm.r>0;
            alpha = (S_01*beta)/(beta'*S_11*beta);
            if isempty(Z2)==0;
               Psi = (M_02/M_22)-(alpha*beta'*(M_12/M_22));
               epsilon = Z0-(alpha*beta'*Z1)-(Psi*Z2);
            else;
               Psi = [];
               epsilon = Z0-(alpha*beta'*Z1);
            end;
            Z = [Z2;beta'*Z1];
         else;
            alpha = [];
            if isempty(Z2)==0;
               Psi = M_02/M_22;
               epsilon = Z0-Psi*Z2;
               Z = Z2;
            else;
               Psi = [];
               epsilon = Z0;
               Z = [];
            end;
         end;
         Omega = (1/T)*epsilon*epsilon';
         if (isempty(Z)==0)&(tm.restrict(1,2)==0);
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
         else;
            Sigma_theta = [];
         end;
         %
         % estimate beta under restrictions
         %
         if tm.restrict(1,2)==1;
            [beta,newlambda,dfbeta,stdbeta,outputtxt,abort,infomat] = betarestrict(S_00,S_11,S_01,S_10,beta,T,tm.modelsdir,tm.modelfile,0,tm.dcoint);
            %
            % new alpha and beta estimates
            %
            alpha = (S_01*beta)/(beta'*S_11*beta);
            if isempty(Z2)==0;
               Psi = (M_02/M_22)-(alpha*beta'*(M_12/M_22));
               Z = [Z2;beta'*Z1];
            else;
               Psi = [];
               Z = beta'*Z1;
            end;
            Theta = [Psi alpha];
            epsilon = Z0-(Theta*Z);
            Omega = (1/T)*epsilon*epsilon';
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
            %
            % estimate alpha under restrictions
            %
            if tm.restrict(1,1)==1;
               file = strcat(tm.modelsdir,'G-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
               G = loaddata(file);
               [Rmat,rvec] = rmatrix(G,tm.n,tm.r);
               dfalpha = size(G,1);
               if isempty(Z2)==0;
                  Rmat = [zeros(dfalpha,size(Z2,1)*tm.n) Rmat];
               end;
               if curr_ini.reestbeta==0;
                  [theta,Sigma_theta,infomat,abort] = linrest(Z0,Z,Theta,Rmat,rvec,0);
               else;
                  [theta,Sigma_theta,beta,stdbeta,Z,infomat,abort] = alpha_beta_restrict(Z0,Z1,Z2,Z,beta,Theta,Rmat,rvec,tm.dcoint,tm.q1,tm.modelsdir,tm.modelfile,0);
               end;
               %
               % fix all the other parameters
               %
               epsilon = Z0-(theta*Z);
               Omega = (1/T)*epsilon*epsilon';
               g2 = size(theta,2);
               alpha = theta(:,g2-tm.r+1:g2);
               if g2>tm.r;
                  Psi = theta(:,1:g2-tm.r);
               else;
                  Psi = [];
               end;
            end;
         end;
         %
         % consider restrictions on C
         %
         if (size(tm.restrict,2)==3)&(tm.restrict(1,3)==1);
            %
            % collect restrictions on C
            %
            file = strcat(tm.modelsdir,'C-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
            Cres = loaddata(file);
            [Rmat_C,rvec_C] = rmatrix(Cres,tm.n,tm.n);
            [theta,Sigma_theta,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,abort] = c_restrict(Z0,Z1,Z2,tm.restrict,beta,[Psi alpha],Rmat_C,rvec_C,tm.p,tm.pe,d_local,tm.dcoint,tm.q0,tm.q1,tm.modelsdir,tm.modelfile,0);
            Z = [];
            if tm.r>0;
               Z = beta'*Z1;
            end;
            Z = [Z2;Z];
            %
            % fix all the other parameters
            %
            epsilon = Z0-(theta*Z);
            Omega = (1/T)*epsilon*epsilon';
            g2 = size(theta,2);
            if tm.r>0;
               alpha = theta(:,g2-tm.r+1:g2);
            else;
               alpha = [];
            end;
            if g2>tm.r;
               Psi = theta(:,1:g2-tm.r);
            else;
               Psi = [];
            end;
         end;
         %
         % if we have an identified model, we need to compute B_0
         %
         if (isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0);
            if curr_ini.method(1,1)==1;
               [status,abort,B_0,S,gamma,Sigma_B0,infomat,R_l,R_mat,rank_test] = vlaar(tm,tm.type,Psi,alpha,beta,epsilon,Z1,Z2,[],1,0);
               epsilon = (B_0/chol(Omega)')*epsilon;
            elseif curr_ini.method(1,2)==1;
               [status,abort,B_0,Psi,alpha,beta,C,A,stdbeta,S,Sigma_theta,Sigma_B0,Sigma_B0_adj,Sigma_A,infomat,R_mat,epsilon_v,num_rest,dC_theta,dB_theta,max_case,rank_test] = warne(tm,tm.type,tm.keep_rest,tm.del_rest,tm.use_psi_rest,Psi,alpha,beta,epsilon,Z0,Z1,Z2,[],1,0);
               epsilon = (B_0/chol(Omega)')*epsilon_v;
            end;
            Omega = B_0*B_0';
         end;
         Sigma_theta = T*Sigma_theta;
         %
         % close down the wait dialog
         %
         if (tm.restrict(1,2)==1)|((isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0));
            set(waitf.text,'UserData','done');
            delete(mywait);
            drawnow;
         end;
      else;
         type_str = 'Within-sample';
         tag_str = 'ws-';
         %
         % we don't need to re-estimate, but instead just collect estimated
         % parameters
         %
         Psi = tm.Psi;
         alpha = tm.alpha;
         if tm.r>0;
            beta = tm.beta(:,1:tm.r);
         else;
            beta = tm.beta;
         end;
         if (isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0);
            Omega = tm.B_0*tm.B_0';
         else;
            Omega = tm.Omega;
         end;
         if (isfield(tm,'Sigma_theta')==1)&(isempty(tm.Sigma_theta)==0);
            Sigma_theta = tm.Sigma_theta;
         else;
            %
            % we need to compute Sigma_theta
            %
            if tm.r>0;
               Z = beta'*tm.Z1;
            else;
               Z = [];
            end;
            if isempty(tm.Z2)==0;
               Z = [tm.Z2;Z];
            end;
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
         end;
         %
         % make sure it converges in probability to a constant matrix
         %
         T = est_sample_end;
         Sigma_theta = tm.T*Sigma_theta;
      end;
      %
      % if tm.dcoint==1 we need to fix Sigma_theta such that its dimensions
      % are correct
      %
      if tm.dcoint==1;

      end;
      %
      % Now we have the parameters, we can now start the forecasting part
      %
      f_type = get(f_cast.popup,'Value');
      %
      % Determine the coefficients of lagged changes of the endogenous variables
      %
      if tm.p>1;
         J_gamma = j_gamma(tm.n,tm.p,tm.pe,tm.r,d_local,tm.q1,tm.q0);
         Gamma = [Psi alpha]*J_gamma';
      end;
      %
      % Determine the coefficients on deterministics and the variable values
      % over the forecast horizon
      %
      if d_local>0;
         mu_mat = Psi(:,1:d_local);
         deter_mat = tm.Z2(1:d_local,f_sample_begin:f_sample_end);
      else;
         mu_mat = [];
         deter_mat = [];
      end;
      %
      if (tm.dcoint>0)&(tm.r>0);
         mu_mat = [mu_mat alpha*beta(tm.n+tm.dcoint+tm.q1,:)'];
         deter_mat = [deter_mat;tm.Z1(tm.n+tm.dcoint+tm.q1,f_sample_begin:f_sample_end)];
      end;
      %
      % do the levels part
      %
      if (f_type==1)|(f_type==3);
         %
         % transform into levels parameters
         %
         ip = zeros(tm.p,tm.p);
         for i=1:tm.p;
            for j=1:tm.p;
               if j>=i;
                  ip(j,i)=1;
               end;
            end;
         end;
         %
         if tm.r>0;
            Pi = alpha*beta(1:tm.n,:)';
         else;
            Pi = zeros(tm.n,tm.n);
         end;
         %
         ip = kron(ip,eye(tm.n));
         J = eye(tm.n);
         if tm.p>=2;
            l_Phi = [(Pi+eye(tm.n)) -Gamma];
            l_Phi = l_Phi/ip;
            l_Phi = [l_Phi;[eye(tm.n*(tm.p-1)) zeros(tm.n*(tm.p-1),tm.n)]];
            J = [J zeros(tm.n,tm.n*(tm.p-1))];
         else;
            l_Phi = (Pi+eye(tm.n))/ip;
         end;
         %
         % setting up the variable vector for the VAR(1) form
         %
         y_vec = tm.x0(tm.p+est_sample_end,:)';
         for i=1:tm.p-1;
            y_vec = [y_vec;tm.x0(tm.p+est_sample_end-i,:)'];
         end;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*l_Phi*y(T+h-1) + mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_vec 
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         %
         % initializing
         %
         l_pred1 = l_Phi*y_vec;
         if isempty(mu_mat)==0;
            l_pred1 = l_pred1+(J'*mu_mat*deter_mat(:,1));
         end;
         l_pred = zeros(tm.n*tm.p,num_f_cast);
         l_pred(:,1) = l_pred1;
         %
         for i=2:num_f_cast;
            l_pred1 = l_Phi*l_pred(:,i-1);
            if isempty(mu_mat)==0;
               l_pred1 = l_pred1+(J'*mu_mat*deter_mat(:,i));
            end;
            l_pred(:,i) = l_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         l_stdev = zeros(tm.n,num_f_cast);
         %
         if tm.p>1;
            ip = inverse(ip);
            ip(1:tm.n,tm.n*(tm.p-1)+1:tm.n*tm.p) = eye(tm.n);
            ip(tm.n*(tm.p-1)+1:tm.n*tm.p,tm.n*(tm.p-1)+1:tm.n*tm.p) = zeros(tm.n,tm.n);
         end;
         if tm.r>0;
            if tm.p==1;
               W = beta(1:tm.n,:);
            else;
               W = [[eye(tm.n*(tm.p-1)) zeros(tm.n*(tm.p-1),tm.r)];[zeros(tm.n,tm.n*(tm.p-1)) beta(1:tm.n,:)]];
            end;
            W = ip*W;
         else;
            if tm.p==1;
               W = [];   % need to remember this case
            else;
               W = [eye(tm.n*(tm.p-1));zeros(tm.n,tm.n*(tm.p-1))];
               W = ip*W;
            end;
         end;
         %
         % now we compute the uncertainty for the first forecast taking parameter uncertainty into account
         %
         if isempty(mu_mat)==0;
            y_deriv1 = kron(deter_mat(:,1)',eye(tm.n));
         else;
            y_deriv1 = [];
         end;
         if isempty(W)==0;
            y_deriv1 = [y_deriv1 kron(y_vec'*W,eye(tm.n))];
         end;
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         l_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).B_i = eye(tm.n);
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = l_Phi;
            else;
               M_i = M_i*l_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).B_i = J*M_i*J';
            Sigma_i = Sigma_i+(m(i).B_i*Omega*m(i).B_i');
            %
            % take parameter uncertainty into account
            %



            %
            % store results
            %
            l_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % keep only the top tm.n rows of the predictions
         %
         l_pred = J*l_pred;
         %
         % Compute forecast errors
         %
         l_errors = tm.x0(tm.p+f_sample_begin:f_sample_end+tm.p,:)'-l_pred;
         %
         % create upper and lower bounds for the forecasts and the forecast
         % errors
         %
         c_val = norm_inv((100+tm.c)/200);
         l_p_upper = l_pred+(c_val*l_stdev);
         l_p_lower = l_pred-(c_val*l_stdev);
         l_e_upper = (c_val*l_stdev);
         l_e_lower = -(c_val*l_stdev);
      end;
      %
      % do the growth part
      %
      if (f_type==2)|(f_type==3);
         J = eye(tm.n);
         P = eye(tm.n);
         if tm.p>1;
            g_Phi = [Gamma alpha];
            if tm.p>2;
               g_Phi = [g_Phi;[eye(tm.n*(tm.p-2)) zeros(tm.n*(tm.p-2),tm.n+tm.r)]];
               J = [J zeros(tm.n,tm.n*(tm.p-2))];
               P = [P;zeros(tm.n*(tm.p-2),tm.n)];
            end;
            if tm.r>0;
               g_Phi = [g_Phi;[beta(1:tm.n,:)'*Gamma (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
         else;
            g_Phi = [zeros(tm.n,tm.n) alpha];
            if tm.r>0;
               g_Phi = [g_Phi;[zeros(tm.r,tm.n) (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
         end;
         %
         % setting up the data vector
         %
         y_tilde = tm.Z0(:,est_sample_end);
         if tm.p>2;
            for i=1:(tm.p-2);
               y_tilde = [y_tilde;tm.Z0(:,est_sample_end-i)];
            end;
         end;
         if tm.r>0;
            y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+est_sample_end,:)')];
         end;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*g_Phi*y(T+h-1) + mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_tilde
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         g_pred1 = g_Phi*y_tilde;
         if isempty(mu_mat)==0;
            g_pred1 = g_pred1+(P*mu_mat*deter_mat(:,1));
         end;
         if tm.p>1;
            g_pred = zeros((tm.n*(tm.p-1))+tm.r,num_f_cast);
         else;
            g_pred = zeros(tm.n+tm.r,num_f_cast);
         end;
         g_pred(:,1) = g_pred1;
         for i=2:num_f_cast;
            g_pred1 = g_Phi*g_pred(:,i-1);
            if isempty(mu_mat)==0;
               g_pred1 = g_pred1+(P*mu_mat*deter_mat(:,i));
            end;
            g_pred(:,i) = g_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         g_stdev = zeros(tm.n,num_f_cast);
         %
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         g_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).G_i = eye(tm.n);
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = g_Phi;
            else;
               M_i = M_i*g_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).G_i = J*M_i*P;
            Sigma_i = Sigma_i+(m(i).G_i*Omega*m(i).G_i');
            %
            % take parameter uncertainty into account
            %


            %
            % store results
            %
            g_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % compute the long run variances
         %
         Phi_big = kron(g_Phi,g_Phi);
         d_Phi = eye(size(Phi_big,1))-Phi_big;
         d_Phi_inv = d_Phi\eye(size(Phi_big,1));
         v_Sigma = d_Phi_inv*vec(P*Omega*P');
         Sigma_y = vecinv(v_Sigma,size(g_Phi,1));
         Sigma_y = J*Sigma_y*J';
         g_lr_stdev = sqrt(abs(diag(Sigma_y)));
         %
         % keep only the top tm.n rows of the predictions
         %
         g_pred = J*g_pred;
         %
         % Compute forecast errors
         %
         g_errors = tm.Z0(:,f_sample_begin:f_sample_end)-g_pred;
         %
         % create upper and lower bounds for the forecasts and the forecast
         % errors
         %
         c_val = norm_inv((100+tm.c)/200);
         g_p_upper = g_pred+(c_val*g_stdev);
         g_p_lower = g_pred-(c_val*g_stdev);
         g_e_upper = (c_val*g_stdev);
         g_e_lower = -(c_val*g_stdev);
         g_lr_upper = c_val*g_lr_stdev;
         g_lr_lower = -c_val*g_lr_stdev;
      end;
      %
      % now we have all the necessary data, do the graphics
      %
      if curr_ini.colors==1;
         colorstr1 = [1 0 0];
         colorstr2 = (1/255)*[255 255 177];
         colorstr3 = [0 0 1];
         colorstr4 = [0 0 1];
         l_style = '-';
      else;
         colorstr1 = [0.3 0.3 0.3];
         colorstr2 = [0.6 0.6 0.6];
         colorstr3 = [0.45 0.45 0.45];
         colorstr4 = [0 0 0];
         l_style = '--';
      end;
      realscrsz = get(0,'ScreenSize');
      pos = [(realscrsz(3)-612)/2 (realscrsz(4)-484)/2];
      f_handles = [];
      for i=1:tm.n;
         %
         % Deal with the levels forecasts
         %
         if (f_type==1)|(f_type==3);
            f_handle = figure('Units','pixels', ...
               'Position',[pos 612 484], ...
               'Menubar','none', ...
               'Toolbar','none', ...
               'NumberTitle','off', ...
               'Tag',['levels-forecast-' tag_str strrep(strtrim(tm.myvars(i,:)),' ','') '-'], ...
               'Visible','off', ...
               'Name',['Levels Forecasts - ' strtrim(tm.myvars(i,:))]);
            f_handles = [f_handles f_handle];
            %
            % default figure properties
            %
            setfigureproperties(f_handle);
            %
            % Fixing position for next graph
            %
            pos = [pos(1,1)+22 pos(1,2)-22];
            if pos(1,1)>(realscrsz(3)-612);
               pos = [0 realscrsz(4)-554];
            end;
            if pos(1,2)>(realscrsz(4)-554);
               pos = [0 realscrsz(4)-554];
            elseif pos(1,2)<32;
               pos = [0 realscrsz(4)-554];
            end;
            %
            % set the levels predictions in the upper plot
            %
            subplot(2,1,1);
            %
            % Using a patch to display the confidence region
            %
            my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[l_p_upper(i,:) reverse(l_p_lower(i,:))],colorstr2);
            set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
            hold('on');
            my_l1 = plot(tm.dataindex(1:f_sample_end,1)',tm.x0(tm.p+1:f_sample_end+tm.p,i)','-','LineWidth',1,'Color',colorstr4,'Tag',strtrim(tm.myvars(i,:)));
            hold('on');
            %
            my_l2 = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',l_pred(i,:),l_style,'LineWidth',1,'Color',colorstr1,'Tag',['forecast-' strtrim(tm.myvars(i,:))]);
            hold('on');
            %
            % if Ylim has a negative min and a positive max, plot a zero line
            %
            set(gca,'XLim',[tm.dataindex(1,1) tm.dataindex(f_sample_end,1)],'Layer','top');
            y_lim = get(gca,'YLim');
            if (y_lim(1)<0)&(y_lim(2)>0);
               plot([tm.dataindex(1,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
               hold('on');
            end;
            %
            % plot a vertical line
            %
            plot(tm.dataindex(f_sample_begin,1)*ones(1,2),y_lim,':','LineWidth',1,'Color',[0.4 0.4 0.4],'Tag','nolegend-1');
            hold('on');
            set(gca,'YLim',y_lim);
            %
            title_str = [type_str ' levels forecasts for ' strtrim(tm.myvars(i,:)) ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
            setaxesfonts('Title',title_str);
            %
            % create a legend
            %
            myleg = legend_fcn([my_l1 my_l2 my_p],'Actual','Forecast','Confidence',2);
            my_axes = gca;
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            hold('off');
            %
            % and now the forecast errors
            %
            subplot(2,1,2);
            %
            % Using a patch to display the confidence region
            %
            my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[l_e_upper(i,:) reverse(l_e_lower(i,:))],colorstr2);
            set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
            hold('on');
            my_axes = [my_axes gca];
            %
            my_le = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',l_errors(i,:),l_style,'LineWidth',1,'Color',colorstr1,'Tag',['forecast-errors-' strtrim(tm.myvars(i,:))]);
            hold('on');
            plot([tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
            hold('on');
            set(gca,'XLim',[tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],'Layer','top');
            %
            title_str = [type_str ' levels forecast errors for ' strtrim(tm.myvars(i,:)) ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
            setaxesfonts('Title',title_str);
            %
            % create a legend
            %
            myleg = legend_fcn([my_le my_p],'Forecast error','Confidence',2);
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            hold('off');
            toolbar_fcn(my_axes);
         end;
         %
         % deal with the growth forecasts
         %
         if (f_type==2)|(f_type==3);
            f_handle = figure('Units','pixels', ...
               'Position',[pos 612 484], ...
               'Menubar','none', ...
               'Toolbar','none', ...
               'NumberTitle','off', ...
               'Tag',['growth-forecast-' tag_str strrep(strtrim(tm.myvars(i,:)),' ','') '-'], ...
               'Visible','off', ...
               'Name',['Growth Forecasts - ' strtrim(tm.myvars(i,:))]);
            f_handles = [f_handles f_handle];
            %
            % default figure properties
            %
            setfigureproperties(f_handle);
            %
            % Fixing position for next graph
            %
            pos = [pos(1,1)+22 pos(1,2)-22];
            if pos(1,1)>(realscrsz(3)-612);
               pos = [0 realscrsz(4)-554];
            end;
            if pos(1,2)>(realscrsz(4)-554);
               pos = [0 realscrsz(4)-554];
            elseif pos(1,2)<32;
               pos = [0 realscrsz(4)-554];
            end;
            %
            % set the growth predictions in the upper plot
            %
            subplot(2,1,1);
            %
            % Using a patch to display the confidence region
            %
            my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[g_p_upper(i,:) reverse(g_p_lower(i,:))],colorstr2);
            set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
            hold('on');
            my_g1 = plot(tm.dataindex(1:f_sample_end,1)',tm.Z0(i,1:f_sample_end),'-','LineWidth',1,'Color',colorstr4,'Tag',['d' strtrim(tm.myvars(i,:))]);
            hold('on');
            %
            my_g2 = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',g_pred(i,:),l_style,'LineWidth',1,'Color',colorstr1,'Tag',['forecast-d' strtrim(tm.myvars(i,:))]);
            hold('on');
            %
            % if Ylim has a negative min and a positive max, plot a zero line
            %
            set(gca,'XLim',[tm.dataindex(1,1) tm.dataindex(f_sample_end,1)],'Layer','top');
            y_lim = get(gca,'YLim');
            if (y_lim(1)<0)&(y_lim(2)>0);
               plot([tm.dataindex(1,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
               hold('on');
            end;
            %
            % plot a vertical line
            %
            plot(tm.dataindex(f_sample_begin,1)*ones(1,2),y_lim,':','LineWidth',1,'Color',[0.4 0.4 0.4],'Tag','nolegend-1');
            hold('on');
            set(gca,'YLim',y_lim);
            %
            title_str = [type_str ' growth forecasts for ' strtrim(tm.myvars(i,:)) ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
            setaxesfonts('Title',title_str);
            %
            % create a legend
            %
            myleg = legend_fcn([my_g1 my_g2 my_p],'Actual','Forecast','Confidence',2);
            my_axes = gca;
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            hold('off');
            %
            % and now the forecast errors
            %
            subplot(2,1,2);
            %
            % Using a patch to display the confidence region
            %
            my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[g_e_upper(i,:) reverse(g_e_lower(i,:))],colorstr2);
            set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
            hold('on');
            my_axes = [my_axes gca];
            %
            my_ge = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',g_errors(i,:),l_style,'LineWidth',1,'Color',colorstr1,'Tag',['ferrors-d' strtrim(tm.myvars(i,:))]);
            hold('on');
            %
            % plot long-run standard deviation upper and lower bounds
            %
            my_gu = plot([tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],g_lr_upper(i,1)*ones(1,2),':','LineWidth',1,'Color',colorstr3);
            hold('on');
            my_gl = plot([tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],g_lr_lower(i,1)*ones(1,2),':','LineWidth',1,'Color',colorstr3);
            hold('on');
            %
            % and the zero-line
            %
            plot([tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
            hold('on');
            set(gca,'XLim',[tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],'Layer','top');
            %
            title_str = [type_str ' growth forecast errors for ' strtrim(tm.myvars(i,:)) ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
            setaxesfonts('Title',title_str);
            %
            % create a legend
            %
            myleg = legend_fcn([my_ge my_gu my_p],'Forecast error','Long-run confidence','Confidence',2);
            setaxesfonts(myleg);
            set(gca,'UserData',myleg);
            hold('off');
            toolbar_fcn(my_axes);
         end;
      end;
      %
      % Show or save graphics
      %

      if curr_ini.external==0;
         set(f_handles,'Visible','on');
      else;
         stats = [];
         file_names = '';
         for i=1:size(f_handles,2);
            file = [tm.outputdir tm.modelfile '-' get(f_handles(1,i),'Tag') num2str(tm.n) '.' num2str(tm.r) '-' strrep(getdate(tm.samplebeg,tm.sampleend,tm.freq,f_sample_begin/tm.T),':','.') '-' strrep(getdate(tm.samplebeg,tm.sampleend,tm.freq,f_sample_end/tm.T),':','.')];
            set(0,'CurrentFigure',f_handles(1,i));
            stats = [stats autosavefigure(f_handles(1,i),file)];
            file_names = strvcat(file_names,file);
         end;
         if isempty(stats)==0;
            for i=1:size(stats,2);
               if stats(i)==0;
                  autoshowfigure(strtrim(file_names(i,:)));
               end;
            end;
         end;
      end;
   case 'checkperiod'
      endp = get(f_cast.end,'Value');
      beginp = get(f_cast.begin,'Value');
      if beginp>endp;
         set(f_cast.end,'Value',beginp);
      end;
   case 'method'
      if gcbo==f_cast.method1;
         set(f_cast.method2,'Value',0);
      elseif gcbo==f_cast.method2;
         set(f_cast.method1,'Value',0);
      end;
      set(gcbo,'Value',1);
   case 'done'
      set(f_cast.done,'UserData','done');
      set(forecastgui,'UserData',f_cast);
   case 'wayout'
      set(f_cast.done,'UserData','done');
%      if strcmp(get(f_cast.show,'UserData'),'done')==1;
%         delete(forecastgui);
%      end;
end;

if nargout==1;
   h = forecastgui;
end;

if strcmp(selector,'wayout')==0;
   curr_ini.models = tm;
   set(f_cast.popup,'UserData',curr_ini);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(forecastgui,'Name') ' ' fix_filepath([pwd '\icons\forecast.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(f_cast.icon,'UserData',pid);
      end;
   end;
   %
   set(forecastgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(forecastgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(f_cast.done);
   end;
   waitfor(f_cast.done,'UserData','done');
end;

%
% end of forecastdlg.m
%
