%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of historical %
%          forecast errors and forecast error decompositions     %
%                                                                %
%          Function: histdecompdlg                               %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       fh: forecast horizon                     %
%                       f_error: forecast errors                 %
%                       f_error_decomp: decomposition of         %
%                         forecast errors into parts explained   %
%                         by the individual shocks               %
%                       c_mod: structure with model info         %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        May 3, 2006                      %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = histdecompdlg(selector,fh,f_error,f_error_decomp,c_mod)

if strcmp(selector,'init')==1;
   t_m = c_mod;
   t_m.fh = fh;
   t_m.f_error = f_error;
   t_m.f_error_decomp = f_error_decomp;
   %
   % setting up the actual sample period
   %
   if strcmp(t_m.freq,'q')==1;
      add_beg = (fh-1)/4;
   elseif strcmp(t_m.freq,'m')==1;
      add_beg = (fh-1)/12;
   else;
      add_beg = fh-1;
   end;
   samplebeg = t_m.samplebeg+add_beg;
   [t_m.dataindex,t_m.samplebeg,t_m.sampleend] = make_dataindex(t_m.freq,samplebeg,t_m.sampleend);
   checkyear = floor(t_m.samplebeg);
   checkperiod = t_m.samplebeg-checkyear;
   t_m.subbeginyear = num2str(checkyear);
   if strcmp(t_m.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      t_m.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(t_m.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      t_m.subbeginperiod = num2str(round(checkperiod));
   end;
   t_m.dataindex = t_m.dataindex';
   %
   % check if t_m.evars is empty and if so generate
   % generic names
   %
   if strcmp(t_m.evars,'')==1;
      for i=1:t_m.n;
         if i==1;
            t_m.evars = 'y_1';
         else;
            t_m.evars = [t_m.evars ' y_' num2str(i)];
         end;
      end;
   end;
   t_m.namevector = vecstr(t_m.evars);
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
else;
   thisexit = 1;
   histdecompgui = findobj('Type','figure','Tag','HistDecompGUI');
   histdecomp = get(histdecompgui,'UserData');
   curr_ini = get(histdecomp.popup,'UserData');
   t_m = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      histdecompgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','histdecompdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-341)/2 (realscrsz(4)-140)/2 341 140], ...
         'Tag','HistDecompGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Historical Decompositions', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','histdecompdlg done', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(histdecompgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 339 130],'Graphics',45,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 80 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.forecastbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      histdecomp.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50 200 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',['Forecast Error Sample:  ' t_m.subbeginyear ':' t_m.subbeginperiod ' - ' t_m.subendyear ':' t_m.subendperiod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');

      %
      % Select variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Select Variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableText');


      histdecomp.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[135 20 80 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',t_m.namevector, ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',1);

      %
      % Create graph button
      %

      histdecomp.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','histdecompdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','Done');

      %
      %  Done button
      %

      histdecomp.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[236 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','histdecompdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(histdecompgui,'UserData',histdecomp,...
         'HandleVisibility','callback');

   case 'showgraph'
      elem = get(histdecomp.popup,'Value');
      var_str = strtrim(t_m.myvars(elem,:));
      f_error = t_m.f_error(elem,:);
      if elem==t_m.n;
         newval = 1;
      else;
         newval = elem+1;
      end;
      set(histdecomp.popup,'Value',newval);
      %
      % select decompositions corresponding to this error
      %
      iden = eye(t_m.n);
      e_vec = iden(elem,:);
      sel_mat = kron(iden,e_vec);
      f_error_decomp = sel_mat*t_m.f_error_decomp;
      if curr_ini.colors==1;
         colorstr = 'b';
         colorstr2 = 'r';
         typestr = '-';
      else;
         colorstr = 'k';
         colorstr2 = [0.5 0.5 0.5];
         typestr = '--';
      end;
      %
      hi_handles = [];
      realscrsz = get(0,'ScreenSize');
      pos = [(realscrsz(3)-512)/2 (realscrsz(4)-384)/2];
      %
      if strcmp(t_m.freq,'q')==1;
         per_str = 'quarter'
      elseif strcmp(t_m.freq,'m')==1;
         per_str = 'month';
      else;
         per_str = 'year';
      end;
      %
      for i=1:t_m.n;
         shock_str = strtrim(t_m.myshocks(i,:));
         hi_handle = figure('Units','pixels', ...
            'Position',[pos 512 384], ...
            'Menubar','none', ...
            'Toolbar','none', ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Tag',['forecast-error-' shock_str], ...
            'Name',[num2str(t_m.fh) ' ' per_str ' forecast error for ' var_str ' - ' shock_str]);
         %
         hi_handles = [hi_handles hi_handle];
         %
         %
         % default figure properties
         %
         setfigureproperties(hi_handle);
         %
         % Fixing position for next graph
         %
         pos = [pos(1,1)+22 pos(1,2)-22];
         if pos(1,1)>(realscrsz(3)-512);
            pos = [0 realscrsz(4)-454];
         end;
         if pos(1,2)>(realscrsz(4)-454);
            pos = [0 realscrsz(4)-454];
         elseif pos(1,2)<32;
            pos = [0 realscrsz(4)-454];
         end;
         %
         % draw a zero-line
         %
         plot([t_m.dataindex(1,1) t_m.dataindex(1,size(t_m.dataindex,2))],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
         %
         actual_error = plot(t_m.dataindex,f_error,'-','LineWidth',1,'Color',colorstr,'Tag',['ferror-' var_str '-' num2str(t_m.fh,'%0.0f')]);
         hold('on');
         shock_error = plot(t_m.dataindex,f_error_decomp(i,:),typestr,'LineWidth',1,'Color',colorstr2,'Tag',['ferror-' var_str '-' shock_str '-' num2str(t_m.fh,'%0.0f')]);
         %
         set(gca,'XLim',[t_m.dataindex(1,1) t_m.dataindex(1,size(t_m.dataindex,2))]);
         %
         % fix the y-axis limits
         %
         y_lim = get(gca,'YLim');
         add_y_lim = (abs(y_lim(2))+abs(y_lim(1)))/20;
         set(gca,'YLim',y_lim+[-add_y_lim add_y_lim]);
         %
         if istexgreek(shock_str)==1;
            this_shock = ['\' shock_str];
         else;
            this_shock = shock_str;
         end;
         title_str = ['Historical decomposition of ' num2str(t_m.fh,'%0.0f') ' ' per_str ' forecast errors in ' var_str ' due to ' this_shock];
         setaxesfonts('Title',title_str);
         %
         myleg = legend_fcn([actual_error shock_error],'Actual error',[this_shock ' error'],1);
         setaxesfonts(myleg);
         set(gca,'UserData',myleg);
         toolbar_fcn(gca);
         hold('off');
      end;
      %
      % Show or save graphics
      %
      if curr_ini.external==0;
         set(hi_handles,'Visible','on');
      else;
         stats = [];
         file_names = '';
         for i=1:size(hi_handles,2);
            file = [t_m.outputdir t_m.modelfile '-' get(hi_handles(1,i),'Tag') num2str(t_m.n) '.' num2str(t_m.r) '.' num2str(t_m.fh)];
            set(0,'CurrentFigure',hi_handles(1,i));
            stats = [stats autosavefigure(hi_handles(1,i),file)];
            file_names = strvcat(file_names,file);
         end;
         if isempty(stats)==0;
            for i=1:size(stats,2);
               if stats(i)==0;
                  autoshowfigure(strtrim(file_names(i,:)));
               end;
            end;
         end;
      end;
   case 'done'
      set(histdecomp.done,'UserData','done');
end;

if nargout==1;
   h = histdecompgui;
end;

if strcmp(selector,'done')==0;
   curr_ini.models = t_m;
   set(histdecomp.popup,'UserData',curr_ini);
   set(histdecompgui,'UserData',histdecomp);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(histdecompgui,'Name') ' ' fix_filepath([pwd '\icons\forecast.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(histdecomp.icon,'UserData',pid);
      end;
   end;
   %
   set(histdecompgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(histdecompgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(histdecomp.done);
   end;
   waitfor(histdecomp.done,'UserData','done');
end;

%
% end of histdecompdlg.m
%
