function mystatus = saveas_fcn( h, name, format )
%SAVEAS Save Figure or model to desired output format.
%   SAVEAS(H,'FILENAME')
%   Will save the Figure or model with handle H to file called FILENAME. 
%   The format of the file is determined from the extension of FILENAME.
%
%   SAVEAS(H,'FILENAME','FORMAT')
%   Will save the Figure or model with handle H to file called FILENAME. 
%   in the format specified by FORMAT. FORMAT can be the same values as
%   extensions of FILENAME. The FILENAME extension does not have to be 
%   the same as FORMAT.  Given FORMAT overrides FILENAME extension.
%
%   Valid options for FORMAT are:
%
%   'fig'  - save figure to a single binary FIG-file.  Reload using OPEN. 
%   'm'    - save figure to binary FIG-file, and produce callable
%            M-file for reload.
%   'mfig' - same as M.
%   'mmat' - save figure to callable M-file as series of creation commands 
%            with param-value pair arguments.  Large data is saved to MAT-file.
%            Note: MMAT Does not support some newer graphics features. Use
%                  this format only when code inspection is the primary goal.
%                  FIG-files support all features, and load more quickly. 
%
%   Allowable options also include devices allowed by PRINT.
%
%   Examples:
%
%   Write current figure to MATLAB fig file
%
%       saveas_fcn(gcf, 'output', 'fig')
%
%   Write current figure to windows bitmap file
%
%       saveas_fcn(gcf, 'output', 'bmp')
%
%   See Also LOAD, SAVE, OPEN, PRINT

%   Copyright 1984-2001 The MathWorks, Inc.
%   $Revision: 1.15 $  $Date: 2001/04/15 11:59:25 $ 

toolbar = get(h,'UserData');
checkcaller = get(toolbar.saveas,'UserData');
if isnumeric(checkcaller)==0;
   checkcaller = 0;
end;
mystatus = 0;
mypos = get(h,'Position');

%Input validation
if nargin < 2 
    error( 'Require handle to Figure/model and filename.' )
end

if ~ishandle(h)
    error( 'Invalid handle.' )
end

if ishghandle_fcn(h)
    while ~isfigure_fcn(h) & ~isempty(h)
        h = get(h,'Parent');
    end
    if isempty(h)
        error('Invalid figure handle.')
    end
else
%    if ~strcmp( 'block_diagram', get_param( h, 'type' ) ) ...
%            & ~strcmp( 'SubSystem', get_param( h, 'blocktype' ) )
%        error( 'Invalid Simulink handle.' )
%    end
end


if ~isstr(name) | isempty(name)
    error( 'Invalid filename.' );
end

% since this is a callback from the figure menu, it is possible that
% uimenufcn will give a bogus filename (one with a * in it) if we get
% any *'s in the filename, error out gracefully.
% might want to generalize this to the same set of chars windows prevents
% in filenames - could be crippling on unix though.
if any(name == '*')
    error( [ 'Invalid filename: ' name ', * is not a valid filename character.'] )
end

% Make sure we can write given file. Note fileparts_fcn returns leaf directory in
% name return argument if given filename is only a path, i.e. 'c:\temp'.
[fp,fn,fe]=fileparts_fcn(name);

% NOTE: this does not allow to say:
% saveas_fcn(gcf, 'foo.', '-fig')
% maybe that's OK...
% william - 11/98
if ~isempty(fe) & strcmp(fe, '.')
    error(['Invalid extension: ' name]);
end
if isempty(fe)
   fe = '.fig';
end
if isempty(fn) 
    error( [ 'Invalid filename: ' name ] );
end
if ~isempty(fp) 
    if exist(fp) ~= 7
        error( [ 'Invalid or missing path: ' name ] )
    end
end

if nargin == 2
    if isempty(fe) | strcmp(fe,'.')
        error( [ 'No format or filename extension specified: ' name ])
    end
    format = fe(2:end); %Do not want the '.'
end

curr_ini = find_inisettings;

%
% if png, jpeg of tif selected, create ps via -dpsc. then convert ps to
% selected format with ghostscript
%

oldformat = '';
if strcmp(format,'png')==1;
   format = 'epsc';
   name = strrep(name,'.png','.eps');
   oldformat = 'png';
elseif strcmp(format,'jpg')==1;
   format = 'epsc';
   name = strrep(name,'.jpg','.eps');
   oldformat = 'jpg';
elseif strcmp(format,'tif')==1;
   format = 'epsc';
   name = strrep(name,'.tif','.eps');
   oldformat = 'tif';
elseif strcmp(format,'pcx')==1;
   format = 'epsc';
   name = strrep(name,'.pcx','.eps');
   oldformat = 'pcx';
elseif strcmp(format,'pbm')==1;
   format = 'epsc';
   name = strrep(name,'.pbm','.eps');
   oldformat = 'pbm';
elseif strcmp(format,'pgm')==1;
   format = 'epsc';
   name = strrep(name,'.pgm','.eps');
   oldformat = 'pgm';
elseif strcmp(format,'ppm')==1;
   format = 'epsc';
   name = strrep(name,'.ppm','.eps');
   oldformat = 'ppm';
elseif strcmp(format,'pdf')==1;
   format = 'epsc';
   name = strrep(name,'.pdf','.eps');
   oldformat = 'pdf';
elseif strcmp(format,'emf')==1;
   format = 'epsc';
   name = strrep(name,'.emf','.eps');
   oldformat = 'emf';
elseif strcmp(format,'wmf')==1;
   format = 'epsc';
   name = strrep(name,'.wmf','.eps');
   oldformat = 'wmf';
elseif strcmp(format,'tex')==1;
   format = 'epsc';
   name = strrep(name,'.tex','.eps');
   oldformat = 'tex';
end;

dontdel = 0;
if strcmp(oldformat,'')==0;
   if fileexist(name)==1;
      dontdel = 1;
      if strcmp(computer,'PCWIN')==1;
         newname = [curr_ini.tempdir 'svar-temp-file' getext(name)];
      else;
         newname = [curr_ini.tempdir 'svar-temp-file' getext(name)];
      end;
      astat = copy_file_fcn(name,newname);
   end;
end;

if strcmp(format,'psc')==1;
   if (strcmp(oldformat,'emf')==1)|(strcmp(oldformat,'wmf')==1)|(strcmp(oldformat,'tex')==1);
      thisstat = ispstoedit;
      if thisstat==0;
         if checkcaller==1;
            txt = ['Structural VAR is unable to locate pstoedit. The selected format,' oldformat ' , is therefore not supported.'];
            about(txt,'information','Bad graphics format...',120,350);
         end;
         mystatus = 1;
         return;
      end;
   else;
      thisstat = isgs;
      if thisstat==0;
         if checkcaller==1;
            txt = ['Structural VAR is unable to locate ghostscript. The selected format,' oldformat ' , is therefore not supported.'];
            about(txt,'information','Bad graphics format...',120,350);
         end;
         mystatus = 1;
         return;
      end;
   end;
end;

%set(toolbar.background,'Visible','off');
set(toolbar.box1,'Visible','off');
%set(toolbar.box2,'Visible','off');
%set(toolbar.new,'Visible','off');
%set(toolbar.open,'Visible','off');
%set(toolbar.ybox,'Visible','off');
%set(toolbar.saveas,'Visible','off');
%set(toolbar.ybox1,'Visible','off');
%set(toolbar.print,'Visible','off');
%set(toolbar.edit,'Visible','off');

addtext_value = get(toolbar.addtext,'State');
if strcmp(addtext_value,'on')==1;
   menu_addtext_fcn;
end;
%set(toolbar.addtext,'Visible','off');

addarrow_value = get(toolbar.addarrow,'State');
if strcmp(addarrow_value,'on')==1;
   menu_addarrow_fcn;
end;
%set(toolbar.addarrow,'Visible','off');

addline_value = strcmp(get(toolbar.addline,'State'),'on');
if addline_value==1;
   menu_addline_fcn;
end;
%set(toolbar.addline,'Visible','off');

if isfield(toolbar,'axesmenu')==1;
   addaxes_str = get(toolbar.axesmenu,'Checked');
   if strcmp(addaxes_str,'on')==1;
      toolbar_addaxes_fcn;
   end;
end;

if isempty(get(toolbar.print,'UserData'))==0;
   axeshandles = get(toolbar.print,'UserData');
else;
   axeshandles = [];
end;

if isempty(axeshandles)==0;
   my_surfhandles = [];
   this_azel = [];
   for i=1:size(axeshandles,2);
      my_surfhandles = [my_surfhandles findobj(axeshandles(1,i),'Type','surface')'];
      this_azel = [this_azel;get(axeshandles(1,i),'View')];
   end;
   if isempty(my_surfhandles)==0;
%      set(toolbar.ybox2,'Visible','off');
%      set(toolbar.rotate,'Visible','off');
%      set(toolbar.rotate_default,'Visible','off');
%      set(toolbar.ybox3,'Visible','off');
%      set(toolbar.left,'Visible','off');
%      set(toolbar.up,'Visible','off');
%      set(toolbar.down,'Visible','off');
%      set(toolbar.right,'Visible','off');

   end;
   this_colorbar = findobj(gcf,'Type','axes','Tag','colorbar')';
   if isempty(this_colorbar)==0;
      axeshandles = [axeshandles this_colorbar];
   end;
   act = get(axeshandles(1,1),'Selected');
   if strcmp(act,'on')==1;
      toolbar_lock_fcn;
   end;
else;
   my_surfhandles = [];
end;

%set(toolbar.ybox4,'Visible','off');

zoomin_value = strcmp(get(toolbar.zoomin,'State'),'on');
if zoomin_value==1;
   menu_zoomin_fcn;
end;
%set(toolbar.zoomin,'Visible','off');
zoomout_value = strcmp(get(toolbar.zoomout,'State'),'on');
if zoomout_value==1;
   menu_zoomout_fcn;
end;
%set(toolbar.zoomout,'Visible','off');

if isfield(toolbar,'rotate3d')==1;
   rotate3d_value = strcmp(get(toolbar.rotate3d,'State'),'on');
   if rotate3d_value==1;
      menu_rotate3d_fcn;
   end;
%   set(toolbar.rotate3d,'Visible','off');
end;

%For some formats we have helper SAVEAS... functions
% make sure format is defined to prevent recursion into this file
%if ~isempty(format) & any( exist( ['saveas' format] ) == [2 3 5 6] )
%    feval( ['saveas' format], h, name )
    
%else    
    %If FORMAT is specified, look first to see if it is an extension
    %we know is that of a PRINT output format. If not, look to see if
    %it is a PRINT supported device format specifier.


if strcmp(format,'fig')==1;
    mystatus = 1;
    if isfield(toolbar,'rotate_default')==1;
       set(toolbar.rotate_default,'UserData',this_azel);
    end;
    if (matlabnumber>=6)&(strcmp(computer,'PCWIN')==1);
       %
       % just in case of path problems with saveas function
       %
       old_path = pwd;
       new_path = getpath(name);
       name = getfilename(name);
       cd(new_path);
       saveas(h,name);
       cd(old_path);
       name = [new_path name];
    else;
       saveas(h,name);
    end;
    if fileexist(name)==1;
       mystatus = 0;
    end;
else;
    [ops,dev,ext] = printtables_fcn;
    i = strmatch( format, ext );
    
    if length(i) >= 1
        %Handle special cases, more then one device, i.e. format='ps'
        i = i(1);
        
    elseif isempty(i)
        i = strmatch( format, dev, 'exact'  );
        if isempty(i)
            i = strmatch( format, dev  );
            if ~isempty(i)
                %Need to handle cases of multiple devices, i.e. format='ljet'
                i = i(1);
            end
        end
    end
    
    %FORMAT is a PRINT support ext or driver
    if isempty(i)
        error( [ 'Unsupported format or extension: ' format ] )
    else

         inv_hardcopy = 'on';
         if strcmp(get(h,'InvertHardcopy'),'off')==1;
            %
            % change background color of figure to white or background color
            % of first axes
            %
            inv_hardcopy = 'off';
            set(h,'InvertHardcopy','on');
         end;
         if strcmp(get(h,'PaperPositionMode'),'manual')==1;
            old_fig_units = get(h,'Units');
            old_paper_units = get(h,'PaperUnits');
            set(h,'Units','points','PaperUnits','points');
            fig_position_points = get(h,'Position');
            fig_paper_position = get(h,'PaperPosition');
            scrsz_points = (72/get(0,'ScreenPixelsPerInch'))*get(0,'ScreenSize');
            new_figure_position = [abs((scrsz_points(3:4)-fig_paper_position(3:4))/2) fig_paper_position(3:4)];
            set(h,'Position',new_figure_position);
            set(h,'Units',old_fig_units);
         end;
        print_fcn( h, name, ['-d' dev{i}] );

        set(h,'Position',mypos,'InvertHardcopy',inv_hardcopy);
        

        if strcmp(get(h,'PaperPositionMode'),'manual')==1;
            set(h,'Units','points');
            set(h,'Position',fig_position_points);
            set(h,'Units',old_fig_units,'PaperUnits',old_paper_units);
        end;
        %
        % Insert proper creator information
        %
        if (strcmp(getext(name),'.eps')==1)|(strcmp(getext(name),'.ps')==1);
           mytext = readtext(name);
           mytext{2} = ['%%Creator: Structural VAR, version ' num2str(myvers,'%0.2f')];
           for i=1:max(size(mytext));
              if isempty(findstr(mytext{i},'MathWorks'))==0;
                 mytext{i} = strrep(mytext{i},'MathWorks','WarneWorks');
              end;
           end;
%           old_units = get(h,'PaperUnits');
%           set(h,'PaperUnits','points');
%           new_p_pos = get(h,'PaperPosition');
%           new_p_size = get(h,'PaperSize');
%           old_fig_pos = get(h,'Position');
%           points_per_pixel = 72/get(0,'ScreenPixelsPerInch');
%           new_fig_pos = points_per_pixel*old_fig_pos;
%           set(h,'PaperUnits',old_units);
%           if strcmp(get(h,'PaperOrientation'),'landscape')==1;
%              new_p_pos = [new_p_pos(2) new_p_size(1)-(new_p_pos(1)+new_p_pos(3)) new_p_pos(4) new_p_pos(3)];
%           end;
%           pp_fig_ratio_2 = new_p_pos(4)/new_fig_pos(4);
%           pp_fig_ratio_1 = new_p_pos(3)/new_fig_pos(3);
%           old_p_pos = new_p_pos;
%           if (strcmp(num2str(abs(new_p_pos(4)-new_p_size(2)),'%0.4f'),'0.0000')==0)&(strcmp(num2str(abs(new_p_pos(4)-new_p_size(1)),'%0.4f'),'0.0000')==0);
%              new_p_pos(4) = new_p_pos(4)-(40*points_per_pixel*pp_fig_ratio_2);
%           end;
%           if (strcmp(num2str(abs(new_p_pos(3)-new_p_size(1)),'%0.4f'),'0.0000')==0)&(strcmp(num2str(abs(new_p_pos(3)-new_p_size(2)),'%0.4f'),'0.0000')==0);
%              new_p_pos(3) = new_p_pos(3)-(40*points_per_pixel*pp_fig_ratio_1);
%           end;
%           if new_p_pos(2)>0;
%              new_p_pos(2) = new_p_pos(2)+(20*points_per_pixel*pp_fig_ratio_2);
%           end;
%           if new_p_pos(1)>0;
%              new_p_pos(1) = new_p_pos(1)+(20*points_per_pixel*pp_fig_ratio_1);
%           end;


           myfid = fopen(name,'wt');
           done_papersize = 0;
           for i=1:size(mytext,2);
              fprintf(myfid,'%s\n',strtrim(mytext{i}));
              if i==8;
                 if strcmp(get(h,'PaperOrientation'),'landscape')==1;
                    fprintf(myfid,'%%%%Orientation: Landscape\n');
                 end;
              end;
              if (done_papersize==0)&(i<size(mytext,2));
                 if strcmp(mytext{i+1},'0 cap')==1;
                    fprintf(myfid,'%%%%PaperSize: %s\n',strrep(get(h,'PaperType'),'us',''));
                    done_papersize = 1;
                 end;
              end;
           end;
           fclose(myfid);
        end;

%        set(h,'Visible','on');
        crop = '';
        if isempty(get(toolbar.open,'UserData'))==0;
           crop = ' -dEPSCrop';
        end;
        if strcmp(oldformat,'png')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#png16m -r100 -sOutputFile#"' strrep(name,'.eps','.png') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#png16m -r100 -sOutputFile#' strrep(name,'.eps','.png') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'jpg')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#jpeg -dJPEGQ#100 -r100 -sOutputFile#"' strrep(name,'.eps','.jpg') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#jpeg -dJPEGQ#100 -r100 -sOutputFile#' strrep(name,'.eps','.jpg') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'tif')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#tiff24nc -r100 -sOutputFile#"' strrep(name,'.eps','.tif') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#tiff24nc -r100 -sOutputFile#' strrep(name,'.eps','.tif') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'pcx')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH ' crop ' -sDEVICE#pcx24b -r100 -sOutputFile#"' strrep(name,'.eps','.pcx') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pcx24b -r100 -sOutputFile#' strrep(name,'.eps','.pcx') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'pbm')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pbm -r100 -sOutputFile#"' strrep(name,'.eps','.pbm') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pbm -r100 -sOutputFile#' strrep(name,'.eps','.pbm') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'pgm')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pgm -r100 -sOutputFile#"' strrep(name,'.eps','.pgm') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pgm -r100 -sOutputFile#' strrep(name,'.eps','.pgm') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'ppm')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#ppm -r100 -sOutputFile#"' strrep(name,'.eps','.ppm') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH ' crop ' -sDEVICE#ppm -r100 -sOutputFile#' strrep(name,'.eps','.ppm') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'pdf')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 gscmd = [fix_filepath(curr_ini.ghostscript) ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pdfwrite -dCompatibilityLevel#' curr_ini.pdfversion ' -sOutputFile#"' strrep(name,'.eps','.pdf') '" "' name '"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(gscmd);
                 delete_file_fcn(name);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    gscmd = [curr_ini.ghostscript ' -q -dNOPAUSE -dBATCH' crop ' -sDEVICE#pdfwrite -dCompatibilityLevel#' curr_ini.pdfversion ' -sOutputFile#' strrep(name,'.eps','.pdf') ' ' name];
                    thisstat = unix(gscmd);
                    nextstat = unix(['rm -f ' name]);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'emf')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 pscmd = [fix_filepath(curr_ini.pstoedit) ' -f "emf" "' name '" "' strrep(name,'.eps','.emf') '">"' getpath(name) 'test.out"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(pscmd);
                 delete_file_fcn(name);
                 delete_file_fcn([getpath(name) 'test.out']);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    pscmd = [curr_ini.pstoedit ' -f "emf" ' name ' ' strrep(name,'.eps','.emf') '>' getpath(name) '/test.out'];
                    thisstat = unix(pscmd);
                    nextstat = unix(['rm -f ' name]);
                    finstat = unix(['rm -f ' getpath(name) '/test.out']);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'wmf')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 pscmd = [fix_filepath(curr_ini.pstoedit) ' -f "wmf" "' name '" "' strrep(name,'.eps','.wmf') '">"' getpath(name) 'test.out"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(pscmd);
                 delete_file_fcn(name);
                 delete_file_fcn([getpath(name) 'test.out']);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    pscmd = [curr_ini.pstoedit ' -f "wmf" ' name ' ' strrep(name,'.eps','.wmf') '>' getpath(name) '/test.out'];
                    thisstat = unix(pscmd);
                    nextstat = unix(['rm -f ' name]);
                    finstat = unix(['rm -f ' getpath(name) '/test.out']);
                 end;
              end;
           end;
        elseif strcmp(oldformat,'tex')==1;
           if strcmp(computer,'PCWIN')==1;
              if fileexist(name)==1;
                 pscmd = [fix_filepath(curr_ini.pstoedit) ' -f "latex2e:integers" "' name '" "' strrep(name,'.eps','.tex') '">"' getpath(name) 'test.out"'];
                 if iscompiled==1;
                    gscmd = ['bin\run_exe.exe ' gscmd];
                 end;
                 thisstat = dos(pscmd);
                 delete_file_fcn(name);
                 delete_file_fcn([getpath(name) 'test.out']);
              end;
           else;
              if isunix==1;
                 if fileexist(name)==1;
                    pscmd = [curr_ini.pstoedit ' -f "latex2e:integers" ' name ' ' strrep(name,'.eps','.tex') '>' getpath(name) '/test.out'];
                    thisstat = unix(pscmd);
                    nextstat = unix(['rm -f ' name]);
                    finstat = unix(['rm -f ' getpath(name) '/test.out']);
                 end;
              end;
           end;
        end;
    end

   if dontdel==1;
      astat = copy_file_fcn(newname,name);
      delete_file_fcn(newname);
   end;
end;

if min(ishandle(toolbar.box1))==1;
%   set(toolbar.background,'Visible','on');
   set(toolbar.saveas,'UserData',0);
   set(toolbar.box1,'Visible','on');
%   set(toolbar.box2,'Visible','on');
%   set(toolbar.new,'Visible','on');
%   set(toolbar.open,'Visible','on');
%   set(toolbar.ybox,'Visible','on');
%   set(toolbar.saveas,'Visible','on');
%   set(toolbar.ybox1,'Visible','on');
%   set(toolbar.print,'Visible','on');
%   set(toolbar.edit,'Visible','on');

   if isfield(toolbar,'axesmenu')==1;
      if strcmp(addaxes_str,'on')==1;
         toolbar_addaxes_fcn;
      end;
   end;

   if strcmp(addtext_value,'on')==1;
      menu_addtext_fcn;
   end;
%   set(toolbar.addtext,'Visible','on');

   if strcmp(addarrow_value,'on')==1;
      menu_addarrow_fcn;
   end;
%   set(toolbar.addarrow,'Visible','on');

   if addline_value==1;
      menu_addline_fcn;
   end;
%   set(toolbar.addline,'Visible','on');

%   if isempty(my_surfhandles)==0;
%      set(toolbar.ybox2,'Visible','on');
   %   set(toolbar.rotate,'Visible','on');
%      set(toolbar.rotate_default,'Visible','on');
   %   set(toolbar.ybox3,'Visible','on');
   %   set(toolbar.left,'Visible','on');
   %   set(toolbar.up,'Visible','on');
   %   set(toolbar.down,'Visible','on');
   %   set(toolbar.right,'Visible','on');
%   end;

%   set(toolbar.ybox4,'Visible','on');

   if zoomin_value==1;
      menu_zoomin_fcn;
   end;
%   set(toolbar.zoomin,'Visible','on');
   if zoomout_value==1;
      menu_zoomout_fcn;
   end;
%   set(toolbar.zoomout,'Visible','on');

   if isfield(toolbar,'rotate3d')==1;
      if rotate3d_value==1;
         menu_rotate3d_fcn;
      end;
%      set(toolbar.rotate3d,'Visible','on');
   end;

   if isempty(axeshandles)==0;
      if strcmp(act,'on')==1;
         toolbar_lock_fcn;
      end;
   end;
end;

if (strcmp(format,'fig')==1)&(curr_ini.show_undo==1);
   file = get(toolbar.alignment,'UserData');
   if direxist(getpath(file))==1;
      return;
      saveas(h,file);
   end;
end;

%
% end of saveas_fcn.m
%
