%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for displaying graphs of forecasts, %
%          forecast uncertainty (including parameter uncertainty)%
%          and the actual outcomes. Forecasts may be either      %
%          within-sample or out-of-sample. In the latter case    %
%          we re-estimate all parameters using the smaller       %
%          sample                                                %
%                                                                %
%          Function: var_forecastdlg                             %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       c_mod: structure with model info         %
%                       restrict: 1x2 indicator vector if        %
%                          restricted estimates of alpha and     %
%                          beta are used                         %
%                                                                %
%          Output:                                               %
%                       h: handle                                %
%                                                                %
%          This version:        May 4, 2006                      %
%          Written by:          Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = var_forecastdlg(selector,c_mod,restrict)

if strcmp(selector,'init')==1;
   tm = c_mod;
   tm.restrict = restrict;
   %
   % create string with variables
   %
   tm.allvars = tm.myvars;
   for i=1:size(tm.myvars,1);
      tm.allvars = strvcat(tm.allvars,[strtrim(tm.myvars(i,:)) '(t)-' strtrim(tm.myvars(i,:)) '(t-1)']);
   end;
   %
   % now we add sums of levels and sums of first diffs if we have quarterly or monthly data
   %
   if (strcmp(tm.freq,'q')==1)|(strcmp(tm.freq,'m')==1);
      if strcmp(tm.freq,'q')==1;
         add_freq = '(t-4)';
         add_sum = '3';
      else;
         add_freq = '(t-12)';
         add_sum = '11';
      end;
      %
      for i=1:size(tm.myvars,1);
         tm.allvars = strvcat(tm.allvars,['sum(0,' add_sum ')'strtrim(tm.myvars(i,:)) '(t-i)']);
      end;
      %
      for i=1:size(tm.myvars,1);
         tm.allvars = strvcat(tm.allvars,[strtrim(tm.myvars(i,:)) '(t)-' strtrim(tm.myvars(i,:)) add_freq]);
      end;
      %
   end;
   %
   % setting up the actual sample period
   %
   [tm.dataindex,tm.samplebeg,tm.sampleend] = make_dataindex(tm.freq,tm.samplebeg,tm.sampleend);
   checkyear = floor(tm.samplebeg);
   checkperiod = tm.samplebeg-checkyear;
   tm.subbeginyear = num2str(checkyear);
   if strcmp(tm.freq,'q')==1;
      checkperiod = 4*(checkperiod+(1/8));
      tm.subbeginperiod = num2str(round(checkperiod));
   elseif strcmp(tm.freq,'m')==1;
      checkperiod = 12*(checkperiod+(1/24));
      tm.subbeginperiod = num2str(round(checkperiod));
   end;
   %
   % create sample strings
   %
   tm.sample_string = '';
   for i=tm.df+4:tm.T;
      tm.sample_string = strvcat(tm.sample_string,getdate(tm.samplebeg,tm.sampleend,tm.freq,i/tm.T));
   end;
   %
   thisexit = 0;
   curr_ini = find_inisettings;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
else;
   thisexit = 1;
   forecastgui = findobj('Type','figure','Tag','VARForecastGUI')';
   f_cast = get(forecastgui,'UserData');
   curr_ini = get(f_cast.popup,'UserData');
   tm = curr_ini.models;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      forecastgui = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','var_forecastdlg.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-420)/2 (realscrsz(4)-263)/2 420 263], ...
         'Tag','VARForecastGUI', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Graph Transformed Variable Forecasts', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','var_forecastdlg wayout', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(forecastgui,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 418 253],'Forecast Selections',96,[0.5 1]);

      imageaxes = axes('Units','pixels', ...
         'Box','on', ...
         'TickLength',[0 0], ...
         'CameraUpVector',[0 1 0], ...
         'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[366 188 39 39], ...
         'XTickLabelMode','manual', ...
         'YTickLabelMode','manual', ...
         'Tag','AxisLogo', ...
         'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

      mycons = getstoredimages;
      if curr_ini.use_toolbar_bgcolor==1;
         bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
         bgcolor(1) = max(0,bgcolor(1));
         bgcolor(2) = max(0,bgcolor(2));
         bgcolor(3) = max(0,bgcolor(3));
      else;
         bgcolor = get(0,'defaultuicontrolbackgroundcolor');
      end;
      %
      image('CData',mirrorimage(maketrans(mycons.forecastbig_image,[160 160 164],bgcolor)),'Clipping','off','Parent',imageaxes);

      %
      % Forecast period selection
      %

      f_cast.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 219 80 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast period:', ...
         'HorizontalAlignment','left', ...
         'Tag','PeriodText');

      f_cast.begin = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 201 130 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.sample_string, ...
         'Callback','var_forecastdlg checkperiod', ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',max(1,size(tm.sample_string,1)-tm.h+1));

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[146 203 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','_', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.end = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[156 201 130 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.sample_string, ...
         'Callback','var_forecastdlg checkperiod', ...
         'HorizontalAlignment','center', ...
         'Tag','GraphListbox',...
         'Value',size(tm.sample_string,1));

      %
      % Select forecast variable
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 166 100 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','FormatText');

      f_cast.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 148 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.allvars, ...
         'Callback','var_forecastdlg checkvar', ...
         'HorizontalAlignment','center', ...
         'Tag','TypePopup',...
         'Value',1);

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[116 146 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','+', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.constant = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[126 148 50 20], ...
         'Style','edit', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','0', ...
         'Callback','var_forecastdlg checkconstant', ...
         'HorizontalAlignment','right', ...
         'Tag','ConstantEdit');

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[176 146 10 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','x', ...
         'HorizontalAlignment','center', ...
         'Tag','DashText');

      f_cast.secondvar = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[186 148 100 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String',tm.allvars, ...
         'Callback','var_forecastdlg checkvar', ...
         'HorizontalAlignment','center', ...
         'Tag','SecondTypePopup',...
         'Value',2);

      %
      % Select transformation function (53 pixels up)
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 113 210 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Transformation function:', ...
         'HorizontalAlignment','left', ...
         'Tag','TransformText');

      f_cast.transform = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[16 95 270 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'String','none|exp(variable)|exp(variable)-1|100*exp(variable)|100*(exp(variable)-1)|4*variable|12*variable|400*variable|1200*variable|(1+variable)^4|(1+variable)^4-1|100*(1+variable)^4|100*((1+variable)^4-1)|(1+variable)^12|(1+variable)^12-1|100*(1+variable)^12|100*((1+variable)^12-1)', ...
         'HorizontalAlignment','center', ...
         'Tag','TransformPopup',...
         'Value',1);

      %
      % Forecast method
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[16 60 100 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String','Forecast method:', ...
         'HorizontalAlignment','left', ...
         'Tag','MethodText');

      f_cast.method1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[16 40 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','radiobutton', ...
         'Value',1, ...
         'Enable','on', ...
         'Visible','on', ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','var_forecastdlg method', ...
      	 'String','Within-sample forecasts', ...
      	 'Tag','WithinSampleButton');

      f_cast.method2 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[16 20 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','radiobutton', ...
         'Value',0, ...
         'Enable','on', ...
         'Visible','on', ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','var_forecastdlg method', ...
      	 'String','Out-of-sample forecasts', ...
      	 'Tag','OutOfSampleButton');

      if curr_ini.outofsample==1;
         set(f_cast.method1,'Value',0);
         set(f_cast.method2,'Value',1);
      end;

      %
      % Create graph button
      %

      f_cast.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[312 55 90 20], ...
         'String','Display', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','var_forecastdlg showgraph', ...
         'Enable','on', ...
         'UserData','working', ...
         'Tag','ShowButton');

      %
      %  Done button
      %

      f_cast.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[312 20 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','var_forecastdlg done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(forecastgui,'UserData',f_cast,...
         'HandleVisibility','callback');

   case 'showgraph'
      %
      % Determine the sample
      %
      f_sample_begin = get(f_cast.begin,'Value')+(tm.df+3);
      f_sample_end = get(f_cast.end,'Value')+(tm.df+3);
      est_sample_end = get(f_cast.begin,'Value')+(tm.df+2);
      d_local = tm.d;
      %
      % check method
      %
      if get(f_cast.method2,'Value')==1;
         type_str = 'Out-of-sample';
         tag_str = 'oos-';
         %
         % need to check if any deterministic variables should be deleted (may need to change locally tm.d)
         %
         if isempty(tm.x1)==0;
            x1 = tm.x1(1:tm.p+est_sample_end,:);
         else;
            x1 = [];
         end;
         if isempty(tm.x1_r)==0;
            x1_r = tm.x1_r(1:tm.p+est_sample_end,:);
         else;
            x1_r = [];
         end;
         x0 = tm.x0(1:tm.p+est_sample_end,:);
         [x1,deter,deterdel] = checkdeter(x1,x1_r,tm.deter);
         d_local = size(x1,2);
         [Z0,Z1,Z2,Z1t] = vecmdata(x0,tm.p,x1);
         %
         if isempty(x1_r)==0;
            num = size(x1_r,1);
            x1_r = x1_r(tm.p+1:num,:);
            Z1 = [Z1;x1_r'];
         end;
         %
         % we need to re-estimate everything
         %
         T = size(Z0,2);
         %
         % Display a dialog to let the user know what is going on unless
         % we're estimating the minimum
         %
         if (tm.restrict(1,2)==1)|((isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0));
            txt = strvcat(' ','Re-estimating parameters.','Please hold!');
            mywait = waitdlg(txt,'information');
            waitf = get(mywait,'UserData');
         end;
         %
         [S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,trace_test] = mlcointbasic(Z0,Z1,Z2,tm.n);
         if tm.r>0;
            beta = beta(:,1:tm.r);
         else;
            beta = [];
         end;
         if tm.r>0;
            alpha = (S_01*beta)/(beta'*S_11*beta);
            if isempty(Z2)==0;
               Psi = (M_02/M_22)-(alpha*beta'*(M_12/M_22));
               epsilon = Z0-(alpha*beta'*Z1)-(Psi*Z2);
            else;
               Psi = [];
               epsilon = Z0-(alpha*beta'*Z1);
            end;
            Z = [Z2;beta'*Z1];
         else;
            alpha = [];
            if isempty(Z2)==0;
               Psi = M_02/M_22;
               epsilon = Z0-Psi*Z2;
               Z = Z2;
            else;
               Psi = [];
               epsilon = Z0;
               Z = [];
            end;
         end;
         Omega = (1/T)*epsilon*epsilon';
         if (isempty(Z)==0)&(tm.restrict(1,2)==0);
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
         else;
            Sigma_theta = [];
         end;
         %
         % estimate beta under restrictions
         %
         if tm.restrict(1,2)==1;
            [beta,newlambda,dfbeta,stdbeta,outputtxt,abort,infomat] = betarestrict(S_00,S_11,S_01,S_10,beta,T,tm.modelsdir,tm.modelfile,0,tm.dcoint);
            %
            % new alpha and beta estimates
            %
            alpha = (S_01*beta)/(beta'*S_11*beta);
            if isempty(Z2)==0;
               Psi = (M_02/M_22)-(alpha*beta'*(M_12/M_22));
               Z = [Z2;beta'*Z1];
            else;
               Psi = [];
               Z = beta'*Z1;
            end;
            Theta = [Psi alpha];
            epsilon = Z0-(Theta*Z);
            Omega = (1/T)*epsilon*epsilon';
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
            %
            % estimate alpha under restrictions
            %
            if tm.restrict(1,1)==1;
               file = strcat(tm.modelsdir,'G-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
               G = loaddata(file);
               [Rmat,rvec] = rmatrix(G,tm.n,tm.r);
               dfalpha = size(G,1);
               if isempty(Z2)==0;
                  Rmat = [zeros(dfalpha,size(Z2,1)*tm.n) Rmat];
               end;
               if curr_ini.reestbeta==0;
                  [theta,Sigma_theta,infomat,abort] = linrest(Z0,Z,Theta,Rmat,rvec,0);
               else;
                  [theta,Sigma_theta,beta,stdbeta,Z,infomat,abort] = alpha_beta_restrict(Z0,Z1,Z2,Z,beta,Theta,Rmat,rvec,tm.dcoint,tm.q1,tm.modelsdir,tm.modelfile,0);
               end;
               %
               % fix all the other parameters
               %
               epsilon = Z0-(theta*Z);
               Omega = (1/T)*epsilon*epsilon';
               g2 = size(theta,2);
               alpha = theta(:,g2-tm.r+1:g2);
               if g2>tm.r;
                  Psi = theta(:,1:g2-tm.r);
               else;
                  Psi = [];
               end;
            end;
         end;
         %
         % consider restrictions on C
         %
         if (size(tm.restrict,2)==3)&(tm.restrict(1,3)==1);
            %
            % collect restrictions on C
            %
            file = strcat(tm.modelsdir,'C-',int2str(tm.n),'x',int2str(tm.r),'-',tm.modelfile,'.rest');
            Cres = loaddata(file);
            [Rmat_C,rvec_C] = rmatrix(Cres,tm.n,tm.n);
            [theta,Sigma_theta,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,abort] = c_restrict(Z0,Z1,Z2,tm.restrict,beta,[Psi alpha],Rmat_C,rvec_C,tm.p,tm.pe,d_local,tm.dcoint,tm.q0,tm.q1,tm.modelsdir,tm.modelfile,0);
            Z = [];
            if tm.r>0;
               Z = beta'*Z1;
            end;
            Z = [Z2;Z];
            %
            % fix all the other parameters
            %
            epsilon = Z0-(theta*Z);
            Omega = (1/T)*epsilon*epsilon';
            g2 = size(theta,2);
            if tm.r>0;
               alpha = theta(:,g2-tm.r+1:g2);
            else;
               alpha = [];
            end;
            if g2>tm.r;
               Psi = theta(:,1:g2-tm.r);
            else;
               Psi = [];
            end;
         end;
         %
         % if we have an identified model, we need to compute B_0
         %
         if (isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0);
            if curr_ini.method(1,1)==1;
               [status,abort,B_0,S,gamma,Sigma_B0,infomat,R_l,R_mat,rank_test] = vlaar(tm,tm.type,Psi,alpha,beta,epsilon,Z1,Z2,[],1,0);
               epsilon = (B_0/chol(Omega)')*epsilon;
            elseif curr_ini.method(1,2)==1;
               [status,abort,B_0,Psi,alpha,beta,C,A,stdbeta,S,Sigma_theta,Sigma_B0,Sigma_B0_adj,Sigma_A,infomat,R_mat,epsilon_v,num_rest,dC_theta,dB_theta,max_case,rank_test] = warne(tm,tm.type,tm.keep_rest,tm.del_rest,tm.use_psi_rest,Psi,alpha,beta,epsilon,Z0,Z1,Z2,[],1,0);
               epsilon = (B_0/chol(Omega)')*epsilon_v;
            end;
            Omega = B_0*B_0';
         end;
         Sigma_theta = T*Sigma_theta;
         %
         % temporary material
         %
%         TR_mat = zeros(6,size(vec([Psi alpha]),1));
%         TR_mat(1,19) = 1;
%         TR_mat(2,25) = 1;
%         TR_mat(3,31) = 1;
%         TR_mat(4,37) = 1;
%         TR_mat(5,43) = 1;
%         TR_mat(6,13) = 1;
%         Tr_vec = zeros(6,1);
%         TZ = [Z2;beta'*Z1];
%         [Theta,Sigma_theta,infmat,dabort] = linrest(Z0,Z,[Psi alpha],TR_mat,Tr_vec);
%         epsilon = Z0-(Theta*TZ);
%         Omega = (1/T)*epsilon*epsilon';
%         g2 = size(Theta,2);
%         alpha = Theta(:,g2-tm.r+1:g2);
%         Psi = Theta(:,1:g2-tm.r);
%         disp(Psi);
         %
         % close down the wait dialog
         %
         if (tm.restrict(1,2)==1)|((isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0));
            set(waitf.text,'UserData','done');
            delete(mywait);
            drawnow;
         end;
      else;
         type_str = 'Within-sample';
         tag_str = 'ws-';
         %
         % we don't need to re-estimate, but instead just collect estimated
         % parameters
         %
         Psi = tm.Psi;
         alpha = tm.alpha;
         if tm.r>0;
            beta = tm.beta(:,1:tm.r);
         else;
            beta = tm.beta;
         end;
         if (isfield(tm,'B_0')==1)&(isempty(tm.B_0)==0);
            Omega = tm.B_0*tm.B_0';
         else;
            Omega = tm.Omega;
         end;
         if (isfield(tm,'Sigma_theta')==1)&(isempty(tm.Sigma_theta)==0);
            Sigma_theta = tm.Sigma_theta;
         else;
            %
            % we need to compute Sigma_theta
            %
            if tm.r>0;
               Z = beta'*tm.Z1;
            else;
               Z = [];
            end;
            if isempty(tm.Z2)==0;
               Z = [tm.Z2;Z];
            end;
            ZZ_inv = (Z*Z')\eye(size(Z,1));
            Sigma_theta = kron(ZZ_inv,Omega);
         end;
         %
         % make sure it converges in probability to a constant matrix
         %
         T = est_sample_end;
         Sigma_theta = tm.T*Sigma_theta;
      end;
      %
      % if tm.dcoint==1 we need to fix Sigma_theta such that its dimensions
      % are correct
      %
      if tm.dcoint==1;

      end;
      %
      % Now we have the parameters, we can now start the forecasting part
      %
      f_var = get(f_cast.popup,'Value');
      s_var = get(f_cast.secondvar,'Value');
      %
      f_sum = 0;
      f_type1 = 0;
      if f_var>tm.n;
         f_vari = f_var-tm.n;
         f_type1 = 1;
         if f_var>2*tm.n;
            f_vari = f_var-(2*tm.n);
            f_type1 = 0;
            f_sum = 1;
            if f_var>3*tm.n;
               f_vari = f_var-(3*tm.n);
               f_type1 = 1;
               f_sum = 2;
            end;
         end;
      else;
         f_vari = f_var;
      end;
      %
      s_sum = 0;
      f_type2 = 0;
      if s_var>tm.n;
         s_vari = s_var-tm.n;
         f_type2 = 1;
         if s_var>(2*tm.n);
            s_vari = s_var-(2*tm.n);
            f_type2 = 0;
            s_sum = 1;
            if s_var>3*tm.n;
               s_vari = s_var-(3*tm.n);
               f_type2 = 1;
               s_sum = 2;
            end;
         end;
      else;
         s_vari = s_var;
      end;
      %
      s_con = strtrim(get(f_cast.constant,'String'));
      ch_div = findstr(s_con,'/');
      if isempty(ch_div)==1;
         s_con = mystr2num(s_con);
      else;
         s_con = strrep(s_con,'/',' ');
         s_con = vecstr(s_con);
         new_val1 = mystr2num(strtrim(s_con(1,:)));
         new_val2 = mystr2num(strtrim(s_con(2,:)));
         s_con = new_val1/new_val2;
      end;
      %
      eye_mat = eye(tm.n);
      e_f = eye_mat(:,f_vari);
      e_s = eye_mat(:,s_vari);
      if s_con==0;
         f_type2 = -1;
      end;
      %
      % Determine the coefficients of lagged changes of the endogenous variables
      %
      if tm.p>1;
         J_gamma = j_gamma(tm.n,tm.p,tm.pe,tm.r,d_local,tm.q1,tm.q0);
         Gamma = [Psi alpha]*J_gamma';
      else;
         Gamma = [];
      end;
      %
      % Determine the coefficients on deterministics and the variable values
      % over the forecast horizon
      %
      if d_local>0;
         mu_mat = Psi(:,1:d_local);
         deter_mat = tm.Z2(1:d_local,f_sample_begin:f_sample_end);
      else;
         mu_mat = [];
         deter_mat = [];
      end;
      %
      if (tm.dcoint>0)&(tm.r>0);
         mu_mat = [mu_mat alpha*beta(tm.n+tm.dcoint+tm.q1,:)'];
         deter_mat = [deter_mat;tm.Z1(tm.n+tm.dcoint+tm.q1,f_sample_begin:f_sample_end)];
      end;
      %
      % do the levels part
      %
      if (f_type1==0)|(f_type2==0);
         %
         % transform into levels parameters
         %
         ip = zeros(tm.p,tm.p);
         for i=1:tm.p;
            for j=1:tm.p;
               if j>=i;
                  ip(j,i)=1;
               end;
            end;
         end;
         %
         if tm.r>0;
            Pi = alpha*beta(1:tm.n,:)';
         else;
            Pi = zeros(tm.n,tm.n);
         end;
         %
         ip = kron(ip,eye(tm.n));
         J = eye(tm.n);
         if tm.p>=2;
            l_Phi = [(Pi+eye(tm.n)) -Gamma];
            l_Phi = l_Phi/ip;
            l_Phi = [l_Phi;[eye(tm.n*(tm.p-1)) zeros(tm.n*(tm.p-1),tm.n)]];
            J = [J zeros(tm.n,tm.n*(tm.p-1))];
         else;
            l_Phi = (Pi+eye(tm.n))/ip;
         end;
         %
         % setting up the variable vector for the VAR(1) form
         %
         y_vec = tm.x0(tm.p+est_sample_end,:)';
         for i=1:tm.p-1;
            y_vec = [y_vec;tm.x0(tm.p+est_sample_end-i,:)'];
         end;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*l_Phi*y(T+h-1) + mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_vec 
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         %
         % initializing
         %
         l_pred1 = l_Phi*y_vec;
         if isempty(mu_mat)==0;
            l_pred1 = l_pred1+(J'*mu_mat*deter_mat(:,1));
         end;
         l_pred = zeros(tm.n*tm.p,num_f_cast);
         l_pred(:,1) = l_pred1;
         %
         for i=2:num_f_cast;
            l_pred1 = l_Phi*l_pred(:,i-1);
            if isempty(mu_mat)==0;
               l_pred1 = l_pred1+(J'*mu_mat*deter_mat(:,i));
            end;
            l_pred(:,i) = l_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         l_stdev = zeros(tm.n,num_f_cast);
         %
         if tm.p>1;
            ip = inverse(ip);
            ip(1:tm.n,tm.n*(tm.p-1)+1:tm.n*tm.p) = eye(tm.n);
            ip(tm.n*(tm.p-1)+1:tm.n*tm.p,tm.n*(tm.p-1)+1:tm.n*tm.p) = zeros(tm.n,tm.n);
         end;
         %
         if tm.r>0;
            if tm.p==1;
               W = beta(1:tm.n,:);
            else;
               W = [[eye(tm.n*(tm.p-1)) zeros(tm.n*(tm.p-1),tm.r)];[zeros(tm.n,tm.n*(tm.p-1)) beta(1:tm.n,:)]];
            end;
            W = ip*W;
         else;
            if tm.p==1;
               W = [];   % need to remember this case
            else;
               W = [eye(tm.n*(tm.p-1));zeros(tm.n,tm.n*(tm.p-1))];
               W = ip*W;
            end;
         end;
         %
         % now we compute the uncertainty for the first forecast taking parameter uncertainty into account
         %
         if isempty(mu_mat)==0;
            y_deriv1 = kron(deter_mat(:,1)',eye(tm.n));
         else;
            y_deriv1 = [];
         end;
         if isempty(W)==0;
            y_deriv1 = [y_deriv1 kron(y_vec'*W,eye(tm.n))];
         end;
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         l_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).B_i = eye(tm.n);
         levels(1).Sigma = Omega;
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = l_Phi;
            else;
               M_i = M_i*l_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).B_i = J*M_i*J';
            Sigma_i = Sigma_i+(m(i).B_i*Omega*m(i).B_i');
            levels(i).Sigma = Sigma_i;
            %
            % take parameter uncertainty into account
            %



            %
            % store results
            %
            l_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % keep only the top tm.n rows of the predictions
         %
         l_pred = J*l_pred;
         %
         % Compute forecast errors
         %
         l_errors = tm.x0(tm.p+f_sample_begin:f_sample_end+tm.p,:)'-l_pred;
      end;
      %
      % do the growth part
      %
      if (f_type1==1)|(f_type2==1);
         J = eye(tm.n);
         P = eye(tm.n);
         if tm.p>1;
            g_Phi = [Gamma alpha];
            if tm.p>2;
               g_Phi = [g_Phi;[eye(tm.n*(tm.p-2)) zeros(tm.n*(tm.p-2),tm.n+tm.r)]];
               J = [J zeros(tm.n,tm.n*(tm.p-2))];
               P = [P;zeros(tm.n*(tm.p-2),tm.n)];
            end;
            if tm.r>0;
               g_Phi = [g_Phi;[beta(1:tm.n,:)'*Gamma (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
         else;
            g_Phi = [zeros(tm.n,tm.n) alpha];
            if tm.r>0;
               g_Phi = [g_Phi;[zeros(tm.r,tm.n) (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
         end;
         %
         % setting up the data vector
         %
         y_tilde = tm.Z0(:,est_sample_end);
         if tm.p>2;
            for i=1:(tm.p-2);
               y_tilde = [y_tilde;tm.Z0(:,est_sample_end-i)];
            end;
         end;
         if tm.r>0;
            y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+est_sample_end,:)')];
         end;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*g_Phi*y(T+h-1) + mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_tilde
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         g_pred1 = g_Phi*y_tilde;
         if isempty(mu_mat)==0;
            g_pred1 = g_pred1+(P*mu_mat*deter_mat(:,1));
         end;
         if tm.p>1;
            g_pred = zeros((tm.n*(tm.p-1))+tm.r,num_f_cast);
         else;
            g_pred = zeros(tm.n+tm.r,num_f_cast);
         end;
         g_pred(:,1) = g_pred1;
         for i=2:num_f_cast;
            g_pred1 = g_Phi*g_pred(:,i-1);
            if isempty(mu_mat)==0;
               g_pred1 = g_pred1+(P*mu_mat*deter_mat(:,i));
            end;
            g_pred(:,i) = g_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         g_stdev = zeros(tm.n,num_f_cast);
         %
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         g_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).G_i = eye(tm.n);
         growth(1).Sigma = Omega;
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = g_Phi;
            else;
               M_i = M_i*g_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).G_i = J*M_i*P;
            Sigma_i = Sigma_i+(m(i).G_i*Omega*m(i).G_i');
            growth(i).Sigma = Sigma_i;
            %
            % take parameter uncertainty into account
            %


            %
            % store results
            %
            g_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % keep only the top tm.n rows of the predictions
         %
         g_pred = J*g_pred;
         %
         % Compute forecast errors
         %
         g_errors = tm.Z0(:,f_sample_begin:f_sample_end)-g_pred;
         %
         % create upper and lower bounds for the forecasts and the forecast
         % errors
         %
      end;
      %
      % do the sum of levels part
      %
      if (f_sum==1)|(s_sum==1);
         %
         % transform into levels parameters
         %
         if strcmp(tm.freq,'q')==1;
            min_p = 4;
         else;
            min_p = 12;
         end;
         %
         p = max(tm.p,min_p);
         ip = zeros(p,p);
         for i=1:p;
            for j=1:p;
               if j>=i;
                  ip(j,i)=1;
               end;
            end;
         end;
         %
         if tm.r>0;
            Pi = alpha*beta(1:tm.n,:)';
         else;
            Pi = zeros(tm.n,tm.n);
         end;
         ip = kron(ip,eye(tm.n));
         J = eye(tm.n);
         if tm.p<min_p;
            nGamma = [Gamma zeros(tm.n,tm.n*(min_p-tm.p))];
         else;
            nGamma = Gamma;
         end;
         ls_Phi = [(Pi+eye(tm.n)) -nGamma];
         ls_Phi = ls_Phi/ip;
         ls_Phi = [ls_Phi;[eye(tm.n*(p-1)) zeros(tm.n*(p-1),tm.n)]];
         J = [J zeros(tm.n,tm.n*(p-1))];
         Kmat = eye(tm.n*p);
         for j=2:min_p;
            Kmat(1:tm.n,tm.n*(j-1)+1:tm.n*j) = eye(tm.n);
         end;
         ls_Phi = (Kmat*ls_Phi)/Kmat;
         %
         % setting up the variable vector for the VAR(1) form
         %
         y_vec = tm.x0(tm.p+est_sample_end,:)';
         for i=1:p-1;
            y_vec = [y_vec;tm.x0(tm.p+est_sample_end-i,:)'];
         end;
         y_vec = Kmat*y_vec;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*ls_Phi*y(T+h-1) + Kmat*mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_vec 
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         %
         % initializing
         %
         ls_pred1 = ls_Phi*y_vec;
         if isempty(mu_mat)==0;
            ls_pred1 = ls_pred1+(Kmat*J'*mu_mat*deter_mat(:,1));
         end;
         ls_pred = zeros(tm.n*p,num_f_cast);
         ls_pred(:,1) = ls_pred1;
         %
         for i=2:num_f_cast;
            ls_pred1 = ls_Phi*ls_pred(:,i-1);
            if isempty(mu_mat)==0;
               ls_pred1 = ls_pred1+(Kmat*J'*mu_mat*deter_mat(:,i));
            end;
            ls_pred(:,i) = ls_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         ls_stdev = zeros(tm.n,num_f_cast);
         %
         if tm.r>0;
            W = [[eye(tm.n*(p-1)) zeros(tm.n*(p-1),tm.r)];[zeros(tm.n,tm.n*(p-1)) beta(1:tm.n,:)]];
            W = ip\W;
         else;
            W = [eye(tm.n*(p-1));zeros(tm.n,tm.n*(p-1))];
            W = ip\W;
         end;
         %
         % now we compute the uncertainty for the first forecast taking parameter uncertainty into account
         %
         if isempty(mu_mat)==0;
            y_deriv1 = kron(deter_mat(:,1)',eye(tm.n));
         else;
            y_deriv1 = [];
         end;
         if isempty(W)==0;
            y_deriv1 = [y_deriv1 kron(y_vec'*W,eye(tm.n))];
         end;
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         ls_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).BS_i = eye(tm.n);
         sumlevels(1).Sigma = Omega;
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = ls_Phi;
            else;
               M_i = M_i*ls_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).BS_i = J*M_i*J';
            Sigma_i = Sigma_i+(m(i).BS_i*Omega*m(i).BS_i');
            sumlevels(i).Sigma = Sigma_i;
            %
            % take parameter uncertainty into account
            %



            %
            % store results
            %
            ls_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % keep only the top tm.n rows of the predictions
         %
         ls_pred = J*ls_pred;
         %
         % Compute forecast errors
         %
         ls_actual = zeros(size(ls_pred));
         for j=f_sample_begin:f_sample_end;
            y_veci = tm.x0(tm.p+j,:)';
            for i=1:p-1;
               y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
            end;
            y_veci = J*Kmat*y_veci;
            ls_actual(:,j-f_sample_begin+1) = y_veci;
         end;
         ls_errors = ls_actual-ls_pred;
         J_lev = J;
         p_lev = p;
      end;
      %
      % do the sum of growth part
      %
      if (f_sum==2)|(s_sum==2);
         %
         % transform into levels parameters
         %
         if strcmp(tm.freq,'q')==1;
            min_p = 4;
         else;
            min_p = 12;
         end;
         %
         p = max(tm.p,min_p);
         J = eye(tm.n);
         P = eye(tm.n);
         if tm.p<min_p+1;
            nGamma = [Gamma zeros(tm.n,tm.n*(min_p+1-tm.p))];
         else;
            nGamma = Gamma;
         end;
         %
         gs_Phi = [nGamma alpha];
         if tm.p>min_p;
            gs_Phi = [gs_Phi;[eye(tm.n*(p-2)) zeros(tm.n*(p-2),tm.n+tm.r)]];
            J = [J zeros(tm.n,tm.n*(p-2))];
            P = [P;zeros(tm.n*(p-2),tm.n)];
            if tm.r>0;
               gs_Phi = [gs_Phi;[beta(1:tm.n,:)'*nGamma (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
            K_mat = eye(tm.n*(p-1)+tm.r);
            for j=2:min_p;
               K_mat(1:tm.n,tm.n*(j-1)+1:tm.n*j) = eye(tm.n);
            end;
            gs_Phi = (K_mat*gs_Phi)/K_mat;
            %
            % setting up the data vector
            %
            y_tilde = tm.Z0(:,est_sample_end);
            for i=1:(p-2);
               y_tilde = [y_tilde;tm.Z0(:,est_sample_end-i)];
            end;
            if tm.r>0;
               y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+est_sample_end,:)')];
            end;
            y_tilde = K_mat*y_tilde;
         else;
            gs_Phi = [gs_Phi;[eye(tm.n*(p-1)) zeros(tm.n*(p-1),tm.n+tm.r)]];
            J = [J zeros(tm.n,tm.n*(p-1))];
            P = [P;zeros(tm.n*(p-1),tm.n)];
            if tm.r>0;
               gs_Phi = [gs_Phi;[beta(1:tm.n,:)'*nGamma (beta(1:tm.n,:)'*alpha)+eye(tm.r)]];
               J = [J zeros(tm.n,tm.r)];
               P = [P;beta(1:tm.n,:)'];
            end;
            K_mat = eye(tm.n*p+tm.r);
            for j=2:min_p;
               K_mat(1:tm.n,tm.n*(j-1)+1:tm.n*j) = eye(tm.n);
            end;
            gs_Phi = (K_mat*gs_Phi)/K_mat;
            %
            % setting up the data vector
            %
            y_tilde = tm.Z0(:,est_sample_end);
            for i=1:(p-1);
               y_tilde = [y_tilde;tm.Z0(:,est_sample_end-i)];
            end;
            if tm.r>0;
               y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+est_sample_end,:)')];
            end;
            y_tilde = K_mat*y_tilde;
         end;
         %
         % The forecasts are now constructed as follows:
         %
         % y(T+h) = J*g_Phi*y(T+h-1) + mu_mat*deter_mat(:,h)
         %
         % where y(T) = y_tilde
         %
         num_f_cast = f_sample_end-f_sample_begin+1;
         gs_pred1 = gs_Phi*y_tilde;
         if isempty(mu_mat)==0;
            gs_pred1 = gs_pred1+(K_mat*P*mu_mat*deter_mat(:,1));
         end;
         gs_pred = zeros(size(gs_pred1,1),num_f_cast);
         gs_pred(:,1) = gs_pred1;
         for i=2:num_f_cast;
            gs_pred1 = gs_Phi*gs_pred(:,i-1);
            if isempty(mu_mat)==0;
               gs_pred1 = gs_pred1+(K_mat*P*mu_mat*deter_mat(:,i));
            end;
            gs_pred(:,i) = gs_pred1;
         end;
         %
         % Dealing with forecast uncertainty
         %
         gs_stdev = zeros(tm.n,num_f_cast);
         %
         Sigma_i = Omega;
         %
         % take parameter uncertainty into account
         %
%         Sigma_i = Sigma_i+((1/T)*y_deriv1*Sigma_theta*y_deriv1');



         gs_stdev(:,1) = sqrt(abs(diag(Sigma_i)));
         m(1).GS_i = eye(tm.n);
         sumgrowth(1).Sigma = Omega;
         %
         % loop over all forecasts
         %
         for i=2:num_f_cast;
            if i==2;
               M_i = gs_Phi;
            else;
               M_i = M_i*gs_Phi;
            end;
            %
            % Store the response matrix in a structure
            %
            m(i).GS_i = J*M_i*P;
            Sigma_i = Sigma_i+(m(i).GS_i*Omega*m(i).GS_i');
            sumgrowth(i).Sigma = Sigma_i;
            %
            % take parameter uncertainty into account
            %


            %
            % store results
            %
            gs_stdev(:,i) = sqrt(abs(diag(Sigma_i)));
         end;
         %
         % keep only the top tm.n rows of the predictions
         %
         gs_pred = J*gs_pred;
         %
         % Compute forecast errors
         %
         gs_actual = zeros(size(gs_pred));
         for j=f_sample_begin:f_sample_end;
            y_tilde = tm.Z0(:,j);
            if tm.p>min_p;
               for i=1:(p-2);
                  y_tilde = [y_tilde;tm.Z0(:,j-i)];
               end;
            else;
               for i=1:(p-1);
                  y_tilde = [y_tilde;tm.Z0(:,j-i)];
               end;
            end;
            if tm.r>0;
               y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
            end;
            y_tilde = J*K_mat*y_tilde;
            gs_actual(:,j-f_sample_begin+1) = y_tilde;
         end;
         %
         gs_errors = gs_actual-gs_pred;
         J_growth = J;
         p_growth = p;
      end;
      %
      % putting things together
      %
      remove_obs = 0;
      if (f_sum==0)&(s_sum==0);
         %
         % No summing of variables
         %
         if (f_type1==0)&(f_type2==0);
            %
            % Adding levels and levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*levels(i).Sigma*e_f)+((s_con*s_con)*e_s'*levels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*levels(i).Sigma*e_s) + (e_s'*levels(i).Sigma*e_f)  );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'+(s_con*e_s'))*l_pred;
            f_var = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'+(s_con*e_s'))*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = [strtrim(tm.myvars(f_vari,:)) '-' strtrim(tm.myvars(s_vari,:))];
            var_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
            var_name_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==1);
            %
            % adding levels and growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).B_i*Omega*m(i).G_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*levels(i).Sigma*e_f)+((s_con*s_con)*e_s'*growth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma*e_s) + (e_s'*combined(i).Sigma'*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*l_pred)+(s_con*e_s'*g_pred);
            f_var1 = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var2 = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'*f_var1)+(s_con*e_s'*f_var2);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = [strtrim(tm.myvars(f_vari,:)) '-d' strtrim(tm.myvars(s_vari,:))];
            var_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==0);
            %
            % adding growth and levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).G_i*Omega*m(i).B_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*growth(i).Sigma*e_f)+((s_con*s_con)*e_s'*levels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*g_pred)+(s_con*e_s'*l_pred);
            f_var1 = tm.Z0(:,1:f_sample_end);
            f_var2 = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'*f_var1)+(s_con*e_s'*f_var2);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:)) '-' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==1);
            %
            % adding growth and growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*growth(i).Sigma*e_f)+((s_con*s_con)*e_s'*growth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*growth(i).Sigma*e_s) + (e_s'*growth(i).Sigma*e_f)  );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'+(s_con*e_s'))*g_pred;
            f_var = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'+(s_con*e_s'))*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:)) '-d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*d' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==-1);
            %
            % just levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*levels(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*l_pred;
            f_var = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = e_f'*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = strtrim(tm.myvars(f_vari,:));
            var_str = var_tag_str;
            var_name_str = var_tag_str;
         elseif (f_type1==1)&(f_type2==-1);
            %
            % just growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*growth(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*g_pred;
            f_var = tm.Z0(:,1:f_sample_end);
            f_var = e_f'*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:))];
            var_name_str = var_tag_str;
         end;
      elseif (f_sum>0)&(s_sum==0);
         %
         % we sum the first variable but not the second
         %
         if (f_type1==0)&(f_type2==0);
            %
            % Adding sums of levels and levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).BS_i*Omega*m(i).B_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumlevels(i).Sigma*e_f)+((s_con*s_con)*e_s'*levels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*ls_pred)+(s_con*e_s'*l_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            %
            l_actual = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'*ls_actual)+(s_con*e_s'*l_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:)) '-' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==1);
            %
            % adding sums of levels and growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).BS_i*Omega*m(i).G_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumlevels(i).Sigma*e_f)+((s_con*s_con)*e_s'*growth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma*e_s) + (e_s'*combined(i).Sigma'*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*ls_pred)+(s_con*e_s'*g_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            g_actual = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'*ls_actual)+(s_con*e_s'*g_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:)) '-d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==0);
            %
            % adding sums of growth and levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).GS_i*Omega*m(i).B_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumgrowth(i).Sigma*e_f)+((s_con*s_con)*e_s'*levels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*gs_pred)+(s_con*e_s'*l_pred);
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            l_actual = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'*gs_actual)+(s_con*e_s'*l_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:)) '-' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==1);
            %
            % adding sums of growth and growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).GS_i*Omega*m(i).G_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumgrowth(i).Sigma*e_f)+((s_con*s_con)*e_s'*growth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*gs_pred)+(s_con*e_s'*g_pred);
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            g_actual = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'*gs_actual)+(s_con*e_s'*g_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:)) '-d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==-1);
            %
            % just sums of levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*sumlevels(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*ls_pred;
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            %
            f_var = e_f'*ls_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:))];
         elseif (f_type1==1)&(f_type2==-1);
            %
            % just sums of growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*sumgrowth(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*gs_pred;
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            f_var = e_f'*gs_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:))];
         end;
      elseif (f_sum==0)&(s_sum>0);
         %
         % we sum the second variable but not the first
         %
         if (f_type1==0)&(f_type2==0);
            %
            % Adding levels and sums levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).B_i*Omega*m(i).BS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*levels(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumlevels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*l_pred)+(s_con*e_s'*ls_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            %
            l_actual = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'*l_actual)+(s_con*e_s'*ls_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = [strtrim(tm.myvars(f_vari,:)) '-sum.' strtrim(tm.myvars(s_vari,:))];
            var_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(s_vari,:))];
            var_name_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==1);
            %
            % adding levels and sums of growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).B_i*Omega*m(i).GS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*levels(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumgrowth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*l_pred)+(s_con*e_s'*gs_pred);
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            l_actual = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = (e_f'*l_actual)+(s_con*e_s'*gs_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = [strtrim(tm.myvars(f_vari,:)) '-sum.d' strtrim(tm.myvars(s_vari,:))];
            var_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = [strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==0);
            %
            % adding growth and sums of levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).G_i*Omega*m(i).BS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*growth(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumlevels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma*e_s) + (e_s'*combined(i).Sigma'*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*g_pred)+(s_con*e_s'*ls_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            g_actual = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'*g_actual)+(s_con*e_s'*ls_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:)) '-sum.' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==1);
            %
            % adding growth and sums of growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).G_i*Omega*m(i).GS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*growth(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumgrowth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*g_pred)+(s_con*e_s'*gs_pred);
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            g_actual = tm.Z0(:,1:f_sample_end);
            f_var = (e_f'*g_actual)+(s_con*e_s'*gs_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:)) '-sum.d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==-1);
            %
            % just levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*levels(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*l_pred;
            f_var = tm.x0(tm.p+1:f_sample_end+tm.p,:)';
            f_var = e_f'*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = strtrim(tm.myvars(f_vari,:));
            var_str = var_tag_str;
            var_name_str = var_tag_str;
         elseif (f_type1==1)&(f_type2==-1);
            %
            % just growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*growth(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*g_pred;
            f_var = tm.Z0(:,1:f_sample_end);
            f_var = e_f'*f_var;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['d' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Delta' strtrim(tm.myvars(f_vari,:))];
            var_name_str = var_tag_str;
         end;
      elseif (f_sum>0)&(s_sum>0);
         %
         % we sum both the first and the second
         %
         if (f_type1==0)&(f_type2==0);
            %
            % Adding sum of levels and sum of levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumlevels(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumlevels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*sumlevels(i).Sigma*e_s) + (e_s'*sumlevels(i).Sigma*e_f)  );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'+(s_con*e_s'))*ls_pred;
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            %
            f_var = (e_f'+(s_con*e_s'))*ls_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:)) '-sum.' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==1);
            %
            % adding sums of levels and sums of growth
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).BS_i*Omega*m(i).GS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumlevels(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumgrowth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma*e_s) + (e_s'*combined(i).Sigma'*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*ls_pred)+(s_con*e_s'*gs_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            remove_obs = max(p_lev-tm.p-1,remove_obs);
            %
            f_var = (e_f'*ls_actual)+(s_con*e_s'*gs_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:)) '-sum.d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==0);
            %
            % adding sums of growth and sums of levels
            %
            combined(1).Sigma = Omega;
            Sigma_i = Omega;
            for i=2:num_f_cast;
               Sigma_i = Sigma_i+(m(i).GS_i*Omega*m(i).BS_i');
               combined(i).Sigma = Sigma_i;
            end;
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumgrowth(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumlevels(i).Sigma*e_s);
               part2 = s_con*( (e_f'*combined(i).Sigma'*e_s) + (e_s'*combined(i).Sigma*e_f) );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'*gs_pred)+(s_con*e_s'*ls_pred);
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            remove_obs = max(p_lev-tm.p-1,remove_obs);
            %
            f_var = (e_f'*gs_actual)+(s_con*e_s'*ls_actual);
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:)) '-sum.' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==1)&(f_type2==1);
            %
            % adding sums of growth and sums of growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = (e_f'*sumgrowth(i).Sigma*e_f)+((s_con*s_con)*e_s'*sumgrowth(i).Sigma*e_s);
               part2 = s_con*( (e_f'*sumgrowth(i).Sigma*e_s) + (e_s'*sumgrowth(i).Sigma*e_f)  );
               fe_std(1,i) = sqrt(abs(part1+part2));
            end;
            f_pred = (e_f'+(s_con*e_s'))*gs_pred;
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            f_var = (e_f'+(s_con*e_s'))*gs_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:)) '-sum.d' strtrim(tm.myvars(s_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(s_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:)) ' + ' num2str(s_con,'%0.2f') '*sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(s_vari,:))];
         elseif (f_type1==0)&(f_type2==-1);
            %
            % just sums of levels
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*sumlevels(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*ls_pred;
            %
            ls_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_veci = tm.x0(tm.p+j,:)';
               for i=1:p_lev-1;
                  if tm.p+j-i>0;
                     y_veci = [y_veci;tm.x0(tm.p+j-i,:)'];
                  else;
                     y_veci = [y_veci;zeros(tm.n,1)];
                  end;
               end;
               y_veci = J_lev*Kmat*y_veci;
               ls_actual(:,j) = y_veci;
            end;
            remove_obs = max(p_lev-tm.p-1,0);
            %
            f_var = e_f'*ls_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}' strtrim(tm.myvars(f_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')' strtrim(tm.myvars(f_vari,:))];
         elseif (f_type1==1)&(f_type2==-1);
            %
            % just sums of growth
            %
            fe_std = zeros(1,num_f_cast);
            for i=1:num_f_cast;
               part1 = e_f'*sumgrowth(i).Sigma*e_f;
               fe_std(1,i) = sqrt(abs(part1));
            end;
            f_pred = e_f'*gs_pred;
            %
            gs_actual = zeros(tm.n,f_sample_end);
            for j=1:f_sample_end;
               y_tilde = tm.Z0(:,j);
               if tm.p>min_p;
                  for i=1:(p_growth-2);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               else;
                  for i=1:(p_growth-1);
                     if j>i;
                        y_tilde = [y_tilde;tm.Z0(:,j-i)];
                     else;
                        y_tilde = [y_tilde;zeros(tm.n,1)];
                        if j==1;
                           remove_obs = remove_obs+1;
                        end;
                     end;
                  end;
               end;
               if tm.r>0;
                  y_tilde = [y_tilde;beta(1:tm.n,:)'*(tm.x0(tm.p+j,:)')];
               end;
               y_tilde = J_growth*K_mat*y_tilde;
               gs_actual(:,j) = y_tilde;
            end;
            %
            f_var = e_f'*gs_actual;
            f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
            var_tag_str = ['sum.d' strtrim(tm.myvars(f_vari,:))];
            var_str = ['\Sigma_{0}^{' num2str(min_p-1,'%0.0f') '}\Delta' strtrim(tm.myvars(f_vari,:))];
            var_name_str = ['sum(0,' num2str(min_p-1,'%0.0f') ')d' strtrim(tm.myvars(f_vari,:))];
         end;
      end;
      %
      c_val = norm_inv((100+tm.c)/200);
      f_upper = f_pred+(c_val*fe_std);
      f_lower = f_pred-(c_val*fe_std);
      e_upper = (c_val*fe_std);
      e_lower = -(c_val*fe_std);
      %
      % check if we should transform the constructed variables
      %
      if get(f_cast.transform,'Value')==2;
         %
         % exponential function
         %
         if (max(max(abs([f_upper;f_lower;f_pred])))>709)|(max(abs(f_var))>709);
            f_var = (1/tm.scale_endo)*f_var;
            f_pred = (1/tm.scale_endo)*f_pred;
            f_upper = (1/tm.scale_endo)*f_upper;
            f_lower = (1/tm.scale_endo)*f_lower;
            setlogtext(['Dividing variables by the factor ' num2str(tm.scale_endo,'%0.4f') ' in graphs...']);
         end;
         f_var = exp(f_var);
         f_pred = exp(f_pred);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = exp(f_upper);
         f_lower = exp(f_lower);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['exp[' var_str ']'];
      elseif get(f_cast.transform,'Value')==3;
         %
         % exponential function minus 1
         %
         if (max(max(abs([f_upper;f_lower;f_pred])))>709)|(max(abs(f_var))>709);
            f_var = (1/tm.scale_endo)*f_var;
            f_pred = (1/tm.scale_endo)*f_pred;
            f_upper = (1/tm.scale_endo)*f_upper;
            f_lower = (1/tm.scale_endo)*f_lower;
            setlogtext(['Dividing variables by the factor ' num2str(tm.scale_endo,'%0.4f') ' in graphs...']);
         end;
         f_var = exp(f_var)-1;
         f_pred = exp(f_pred)-1;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = exp(f_upper)-1;
         f_lower = exp(f_lower)-1;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['exp[' var_str ']-1'];
      elseif get(f_cast.transform,'Value')==4;
         %
         % 100 times exponential function
         %
         if (max(max(abs([f_upper;f_lower;f_pred])))>709)|(max(abs(f_var))>709);
            f_var = (1/tm.scale_endo)*f_var;
            f_pred = (1/tm.scale_endo)*f_pred;
            f_upper = (1/tm.scale_endo)*f_upper;
            f_lower = (1/tm.scale_endo)*f_lower;
            setlogtext(['Dividing variables by the factor ' num2str(tm.scale_endo,'%0.4f') ' in graphs...']);
         end;
         f_var = 100*exp(f_var);
         f_pred = 100*exp(f_pred);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*exp(f_upper);
         f_lower = 100*exp(f_lower);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*exp[' var_str ']'];
      elseif get(f_cast.transform,'Value')==5;
         %
         % 100 times exponential function minus 1
         %
         if (max(max(abs([f_upper;f_lower;f_pred])))>709)|(max(abs(f_var))>709);
            f_var = (1/tm.scale_endo)*f_var;
            f_pred = (1/tm.scale_endo)*f_pred;
            f_upper = (1/tm.scale_endo)*f_upper;
            f_lower = (1/tm.scale_endo)*f_lower;
            setlogtext(['Dividing variables by the factor ' num2str(tm.scale_endo,'%0.4f') ' in graphs...']);
         end;         f_var = 100*(exp(f_var)-1);
         f_pred = 100*(exp(f_pred)-1);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*(exp(f_upper)-1);
         f_lower = 100*(exp(f_lower)-1);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*(exp[' var_str ']-1)'];
      elseif get(f_cast.transform,'Value')==6;
         %
         % 4 times variable
         %
         f_var = 4*f_var;
         f_pred = 4*f_pred;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 4*f_upper;
         f_lower = 4*f_lower;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['4*[' var_str ']'];
      elseif get(f_cast.transform,'Value')==7;
         %
         % 12 times variable
         %
         f_var = 12*f_var;
         f_pred = 12*f_pred;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 12*f_upper;
         f_lower = 12*f_lower;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['12*[' var_str ']'];
      elseif get(f_cast.transform,'Value')==8;
         %
         % 400 times variable
         %
         f_var = 400*f_var;
         f_pred = 400*f_pred;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 400*f_upper;
         f_lower = 400*f_lower;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['400*[' var_str ']'];
      elseif get(f_cast.transform,'Value')==9;
         %
         % 1200 times variable
         %
         f_var = 1200*f_var;
         f_pred = 1200*f_pred;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 1200*f_upper;
         f_lower = 1200*f_lower;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['1200*[' var_str ']'];
      elseif get(f_cast.transform,'Value')==10;
         %
         % (1+variable)^4
         %
         f_var = (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var);
         f_pred = (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper);
         f_lower = (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['(1+[' var_str '])^4'];
      elseif get(f_cast.transform,'Value')==11;
         %
         % (1+variable)^4 - 1
         %
         f_var = ((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var))-1;
         f_pred = ((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred))-1;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = ((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper))-1;
         f_lower = ((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower))-1;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['(1+[' var_str '])^4-1'];
      elseif get(f_cast.transform,'Value')==12;
         %
         % 100*(1+variable)^4
         %
         f_var = 100*((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var));
         f_pred = 100*((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred));
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper));
         f_lower = 100*((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower));
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*(1+[' var_str '])^4'];
      elseif get(f_cast.transform,'Value')==13;
         %
         % 100 * ((1+variable)^4 - 1)
         %
         f_var = 100*(((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var))-1);
         f_pred = 100*(((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred))-1);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*(((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper))-1);
         f_lower = 100*(((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower))-1);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*[(1+[' var_str '])^4-1]'];
      elseif get(f_cast.transform,'Value')==14;
         %
         % (1+variable)^12
         %
         f_var = (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var);
         f_pred = (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper);
         f_lower = (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['(1+[' var_str '])^12'];
      elseif get(f_cast.transform,'Value')==15;
         %
         % (1+variable)^12 - 1
         %
         f_var = ((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var))-1;
         f_pred = ((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred))-1;
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = ((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper))-1;
         f_lower = ((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower))-1;
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['(1+[' var_str '])^12-1'];
      elseif get(f_cast.transform,'Value')==16;
         %
         % 100*(1+variable)^12
         %
         f_var = 100*((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var));
         f_pred = 100*((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred));
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper));
         f_lower = 100*((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower));
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*(1+[' var_str '])^12'];
      elseif get(f_cast.transform,'Value')==17;
         %
         % 100 * ((1+variable)^12 - 1)
         %
         f_var = 100*(((1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var) .* (1+f_var))-1);
         f_pred = 100*(((1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred) .* (1+f_pred))-1);
         f_errors = f_var(1,f_sample_begin:f_sample_end)-f_pred;
         f_upper = 100*(((1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper) .* (1+f_upper))-1);
         f_lower = 100*(((1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower) .* (1+f_lower))-1);
         e_upper = f_upper-f_pred;
         e_lower = f_lower-f_pred;
         var_str = ['100*[(1+[' var_str '])^12-1]'];
      end;
      %
      % now we have all the necessary data, do the graphics
      %
      if curr_ini.colors==1;
         colorstr1 = [1 0 0];
         colorstr2 = (1/255)*[255 255 177];
         colorstr3 = [0 0 1];
         colorstr4 = [0 0 1];
         l_style = '-';
      else;
         colorstr1 = [0.3 0.3 0.3];
         colorstr2 = [0.6 0.6 0.6];
         colorstr3 = [0.45 0.45 0.45];
         colorstr4 = [0 0 0];
         l_style = '--';
      end;
      realscrsz = get(0,'ScreenSize');
      pos = [(realscrsz(3)-612)/2 (realscrsz(4)-484)/2];
      %
      % Display results
      %
      f_handles = figure('Units','pixels', ...
         'Position',[pos 612 484], ...
         'Menubar','none', ...
         'Toolbar','none', ...
         'NumberTitle','off', ...
         'Tag',['transformed-forecast-' tag_str var_tag_str '-'], ...
         'Visible','off', ...
         'Name',['Transformed Forecasts - ' var_name_str]);
      %
      % default figure properties
      %
      setfigureproperties(f_handles);
      %
      % set the levels predictions in the upper plot
      %
      subplot(2,1,1);
      %
      % Using a patch to display the confidence region
      %
      my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[f_upper reverse(f_lower)],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      my_l1 = plot(tm.dataindex(1+remove_obs:f_sample_end,1)',f_var(1,1+remove_obs:f_sample_end),'-','LineWidth',1,'Color',colorstr4,'Tag',var_str);
      hold('on');
      %
      my_l2 = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',f_pred,l_style,'LineWidth',1,'Color',colorstr1,'Tag',['forecast-' var_str]);
      hold('on');
      %
      % if Ylim has a negative min and a positive max, plot a zero line
      %
      set(gca,'XLim',[tm.dataindex(1+remove_obs,1) tm.dataindex(f_sample_end,1)],'Layer','top');
      y_lim = get(gca,'YLim');
      if (y_lim(1)<0)&(y_lim(2)>0);
         plot([tm.dataindex(1,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
         hold('on');
      end;
      %
      % plot a vertical line
      %
      plot(tm.dataindex(f_sample_begin,1)*ones(1,2),y_lim,':','LineWidth',1,'Color',[0.4 0.4 0.4],'Tag','nolegend-1');
      hold('on');
      set(gca,'YLim',y_lim);
      %
      title_str = [type_str ' forecasts for ' var_str ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
      setaxesfonts('Title',title_str);
      %
      % create a legend
      %
      myleg = legend_fcn([my_l1 my_l2 my_p],'Actual','Forecast','Confidence',2);
      my_axes = gca;
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      hold('off');
      %
      % and now the forecast errors
      %
      subplot(2,1,2);
      %
      % Using a patch to display the confidence region
      %
      my_p = patch([tm.dataindex(f_sample_begin:f_sample_end,1)' reverse(tm.dataindex(f_sample_begin:f_sample_end,1)')],[e_upper reverse(e_lower)],colorstr2);
      set(my_p,'FaceColor',colorstr2,'EdgeColor',colorstr2,'Tag','confpatch','LineWidth',1);
      hold('on');
      my_axes = [my_axes gca];
      %
      my_le = plot(tm.dataindex(f_sample_begin:f_sample_end,1)',f_errors,l_style,'LineWidth',1,'Color',colorstr1,'Tag',['ferror-' var_str]);
      hold('on');
      plot([tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],[0 0],'-','LineWidth',1,'Color',[0 0 0]);
      hold('on');
      set(gca,'XLim',[tm.dataindex(f_sample_begin,1) tm.dataindex(f_sample_end,1)],'Layer','top');
      %
      title_str = [type_str ' forecast errors for ' var_str ' with ' num2str(tm.c,'%0.0f') ' % confidence region'];
      setaxesfonts('Title',title_str);
      %
      % create a legend
      %
      myleg = legend_fcn([my_le my_p],'Forecast error','Confidence',2);
      setaxesfonts(myleg);
      set(gca,'UserData',myleg);
      hold('off');
      toolbar_fcn(my_axes);
      %
      % Show or save graphics
      %

      if curr_ini.external==0;
         set(f_handles,'Visible','on');
      else;
         stats = [];
         file_names = '';
         for i=1:size(f_handles,2);
            file = [tm.outputdir tm.modelfile '-' get(f_handles(1,i),'Tag') num2str(tm.n) '.' num2str(tm.r) '-' strrep(getdate(tm.samplebeg,tm.sampleend,tm.freq,f_sample_begin/tm.T),':','.') '-' strrep(getdate(tm.samplebeg,tm.sampleend,tm.freq,f_sample_end/tm.T),':','.')];
            set(0,'CurrentFigure',f_handles(1,i));
            stats = [stats autosavefigure(f_handles(1,i),file)];
            file_names = strvcat(file_names,file);
         end;
         if isempty(stats)==0;
            for i=1:size(stats,2);
               if stats(i)==0;
                  autoshowfigure(strtrim(file_names(i,:)));
               end;
            end;
         end;
      end;
   case 'checkperiod'
      endp = get(f_cast.end,'Value');
      beginp = get(f_cast.begin,'Value');
      if beginp>endp;
         set(f_cast.end,'Value',beginp);
      end;
   case 'checkvar'
      f_var = get(f_cast.popup,'Value');
      s_var = get(f_cast.secondvar,'Value');
      if f_var==s_var;
         var_num = size(get(f_cast.popup,'String'),1);
         if s_var==var_num;
            num = 1;
         else;
            num = s_var+1;
         end;
         set(f_cast.secondvar,'Value',num);
      end;
   case 'checkconstant'
%      new_val = mystr2num(get(f_cast.constant,'String'));
      new_val = strtrim(get(f_cast.constant,'String'));
      ch_div = findstr(new_val,'/');
      if isempty(ch_div)==1;
         new_val = mystr2num(new_val);
      else;
         new_val = strrep(new_val,'/',' ');
         new_val = vecstr(new_val);
         if size(new_val,1)==2;
            new_val1 = mystr2num(strtrim(new_val(1,:)));
            new_val2 = mystr2num(strtrim(new_val(2,:)));
            if isalmostequal(new_val2,0)==0;
               new_val = new_val1/new_val2;
            else;
               new_val = [];
            end;
         else;
            new_val = [];
         end;
      end;
      if isempty(new_val)==1;
         set(f_cast.constant,'String','0');
      end;
   case 'method'
      if gcbo==f_cast.method1;
         set(f_cast.method2,'Value',0);
      elseif gcbo==f_cast.method2;
         set(f_cast.method1,'Value',0);
      end;
      set(gcbo,'Value',1);
   case 'done'
      set(f_cast.done,'UserData','done');
      set(forecastgui,'UserData',f_cast);
   case 'wayout'
      set(f_cast.done,'UserData','done');
%      if strcmp(get(f_cast.show,'UserData'),'done')==1;
%         killguipid(forecastgui);
%      end;
end;

if nargout==1;
   h = forecastgui;
end;

if strcmp(selector,'wayout')==0;
   curr_ini.models = tm;
   set(f_cast.popup,'UserData',curr_ini);
end;

if thisexit==0;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(forecastgui,'Name') ' ' fix_filepath([pwd '\icons\forecast.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(f_cast.icon,'UserData',pid);
      end;
   end;
   %
   set(forecastgui,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(forecastgui,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(f_cast.done);
   end;
   waitfor(f_cast.done,'UserData','done');
end;

%
% end of var_forecastdlg.m
%
