%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for creating data matrices            %
%          for VAR model in levels form                          %
%                                                                %
%          Function: vardata                                     %
%                                                                %
%          Input:                                                %
%                        x0: Txn containing all endogenous       %
%                           variables in levels                  %
%                        p: the lag length for the endogenous    %
%                           variables                            %
%                        x1: Txd containing all deterministic    %
%                           variables                            %
%                        x2: Txq containing all exogenous        %
%                           I(1) and I(0) variables in levels    %
%                        q1: the number of I(1) varibles in x2   %
%                            ordered in the first q1 columns     %
%                        pe: lag length for exogenous I(1) and   %
%                           I(0) variables                       %
%                                                                %
%          Output:                                               %
%                        X0: nx(T-p) containing all endogenous   %
%                            variables in levels                 %
%                        X1: nmx(T-p) containing deterministic   %
%                            variables, lags of endogenous,      %
%                            current values and lags of          %
%                            exogenous variables with I(1)       %
%                            ordered first and I(0) after the    %
%                            I(1) lags                           %
%                                                                %
%          This version: 	      April 7, 2005                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [X0,X1] = vardata(x0,p,x1,x2,q1,pe)

T = size(x0,1);
n = size(x0,2);

%
% Creating the vector of deterministic variables
% If none are included in the function, a vector
% of constants is created
%

if size(x1,1)>0;
   d = size(x1,2);
else;
   d = 0;
end;

%
% Checking if there are any exogenous stochastic
% variables
%

if nargin>=4;
   q = size(x2,2);
else;
   q = 0;
   pe = 0;
end;

if nargin<5;
   q1 = 0;
   q2 = 0;
end;

if nargin<6;
   pe = 0;
end;

if size(x1,1)>0;
   X1 = x1(max(p,pe)+1:T,:)';
else;
   X1 = [];
end;

if q>0;
   q2=q-q1;
end;

%
% Creating the output matrices
%

X0 = x0(max(p,pe)+1:T,:)';

for i=1:p;
   X1 = [X1;x0(max(p,pe)+1-i:T-i,:)'];
end;
if q1>0;
   for i=0:pe;
      X1 = [X1;x2(max(p,pe)+1-i:T-i,1:q1)'];
   end;
end;
if q2>0;
   for i=0:pe;
      X1 = [X1;x2(max(p,pe)+1-i:T-i,q1+1:q)'];
   end;
end;

%
% end of function vardata.m
%
