%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes Wald tests for multi-step Granger            %
%          noncausality conditional on beta - assumes            %
%          chi^2 limiting distribution, requires n>2 and         %
%          r>0 or p>1                                            %
%                                                                %
%          Function: waldmgctest                                 %
%                                                                %
%          Input:                                                %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       p: number of lags for endogenous         %
%                           variables                            %
%                       pe: number of lags for exogenous         %
%                           variables                            %
%                       epsilon: nxT matrix with residuals       %
%                       Z1: nqxT matrix                          %
%                       Z2: nrxT matrix                          %
%                       q1: number of exogenous I(1) variables   %
%                       q0: number of exogenous I(0) variables   %
%                       d: number of unrestricted deterministic  %
%                           variables                            %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                       ind: 0-1 indicator to use restrictions   %
%                           on alpha under the null and the      %
%                           alternative (default=0)              %
%                       Sigma_theta: covariance matrix for Psi   %
%                           and alpha (optional)                 %
%                       c_ind: indicator if restrictions         %
%                           on C have been used                  %
%                       sscorrect: 0-1 indicator if F-test       %
%                           should be computed                   %
%                       var_i and var_j: var_i is Granger        %
%                           noncausal for var_j under the null   %
%                                                                %
%          Output:                                               %
%          	            tests: vector with Wald-type tests       %
%                       df: matrix with degrees of freedom       %
%                       pvals: vector with p-values              %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [tests,df,pvals] = waldmgctest(Psi,alpha,beta,p,pe,epsilon,Z1,Z2,q1,q0,d,t,ind,Sigma_theta,c_ind,sscorrect,noise_parm,var_i,var_j)

tests = zeros(p+1,1);
df = zeros(p+1,1+sscorrect);
pvals = zeros(p+1,1);

%
% Preliminaries
%

[n T] = size(epsilon);

if isempty(alpha)==0;
   r = size(alpha,2);
   w = [Z2;beta'*Z1];
else;
   r = 0;
   beta = [];
   w = Z2;
end;
Omega = (1/T)*epsilon*epsilon';
if isempty(Sigma_theta)==1;
   inv_ww = inverse((w*w'));
   Sigma_theta = kron(inv_ww,Omega);
end;
%
% Take the elements of Sigma_theta that correspond to Gamma_1, ..., Gamma_p-1, alpha
%
J_mat = kron([j_gamma(n,p,pe,r,d,q1,q0);j_alpha(n,p,pe,r,d,q1,q0)],eye(n));
%
% need to verify that J_mat and Sigma_theta are compatible for all cases
%
Sigma_pi = J_mat*Sigma_theta*J_mat';

if r>0;
   Pi = alpha*beta(1:n,:)';
else;
   Pi = zeros(n,n);
end;
if p>=2;
   Phi = Psi(:,d+1:d+n*(p-1));
end;

ip = zeros(p,p);
for i=1:p;
   for j=1:p;
      if j>=i;
         ip(j,i)=1;
      end;
   end;
end;

%
% Computing levels VAR coefficients from
% VECM coefficients
%

ip = kron(ip,eye(n));
J = eye(n);
if p>=2;
   M = [(Pi+eye(n)) -Phi];
   M = M/ip;
   M = [M;[eye(n*(p-1)) zeros(n*(p-1),n)]];
   J = [J zeros(n,n*(p-1))];
else;
   M = (Pi+eye(n))/ip;
end;
Theta = J*M;

%
% compute W for the partials dvec[A(1)...A(p)]/dvec[Gamma(1)...Gamma(p-1) alpha]' = kron(W,eye(n))
%


if p>1;
   ip = inverse(ip);
   ip(1:n,n*(p-1)+1:n*p) = eye(n);
   ip(n*(p-1)+1:n*p,n*(p-1)+1:n*p) = zeros(n,n);
end;
if r>0;
   if p==1;
      W = beta(1:n,:);
   else;
      W = [[eye(n*(p-1)) zeros(n*(p-1),r)];[zeros(n,n*(p-1)) beta(1:n,:)]];
   end;
   W = ip*W;
else;
   if p==1;
      W = [];   % this case shouldn't occur since the function is only available when r>0 or p>1 (and n>2)
   else;
      W = [eye(n*(p-1));zeros(n,n*(p-1))];
      W = ip*W;
   end;
end;

%
% setting up the basic restrictions
%

In = eye(n);
e_i = In(:,var_i);
e_j = In(:,var_j);
H_i = kron(eye(p),e_i);

R_base = kron(H_i',e_j');

%
% computing the restrictions parameters
%

h(1).alpha = J*M;
h(1).M = M;
for i=2:p+1;
   h(i).alpha = h(i-1).alpha*M;
   h(i).M = h(i-1).M*M;
end;

%
% computing the partials of h(i).alpha wrt h(1).alpha
%

h(1).partial = eye(n*n*p);
for j=2:p+1;
   h_partial = kron(h(j-1).M',eye(n));
   for i=1:j-2;
      h_partial = h_partial+kron(h(j-1-i).M',J*h(i).M*J');
   end;
   h_partial = h_partial+kron(eye(n*p),J*h(j-1).M*J');
   h(j).partial = h_partial;
end;

%
% computing the asymptotic covariance matrix for the levels VAR from the
% VEC representation
%

Sigma_alpha = kron(W,eye(n))*Sigma_pi*kron(W',eye(n));

%
% check R_base to make sure that some restrictions are not redundant (only needed if r=0)
%

if r==0;
   ch_R_base = R_base*Sigma_alpha*R_base';
   [red_rest,use_rest] = getdelvars(ch_R_base,[]);
   R_base = R_base(use_rest,:);
end;

%
% computing the tests
%

alpha_mat = [];
R_partial = [];
for i=1:p+1;
   R_parti = R_base*h(i).partial;
   R_partial = [R_partial;R_parti];
   if i==1;
      R_mat = R_base;
      cov_noise =zeros(size(R_mat),size(R_mat));
   else;
      R_mat = [R_mat zeros(size(R_mat,1),size(R_base,2));zeros(size(R_base,1),size(R_mat,2)) R_base];
      diag_mat = T*diag(diag(R_parti*Sigma_alpha*R_parti'));
      cov_noise =[cov_noise zeros(size(cov_noise,1),size(diag_mat,2));zeros(size(diag_mat,1),size(cov_noise,2)) diag_mat];
   end;
   %
   % check which restrictions are independent
   %
   if (ind==1)|(c_ind==1);
      %
      % check the covariance matrix
      %
      sig_check = R_partial*Sigma_alpha*R_partial';
      if rank(sig_check,1.0e-012)==0;
         calc_test = 0;
      else;
         [delrest,keeprest] = getdelvars(sig_check,[]);
         R_partial = R_partial(keeprest,:);
         R_mat = R_mat(keeprest,:);
         %
         % perform an extra check
         %
         sig_check = R_partial*Sigma_alpha*R_partial';
         if rank(sig_check,1.0e-012)==size(sig_check,1);
            calc_test = 1;
         else;
            calc_test = 0;
         end;
      end;
   else;
      keeprest = (1:1:size(R_mat,1))';
      calc_test = 1;
   end;
   alpha_mat = [alpha_mat;vec(h(i).alpha)];
   if calc_test==1;
      %
      % add noise to the test
      %
      noise_cov = noise_parm*cov_noise(keeprest,keeprest);
      if i>1;
         %
         % generate random vector with covariance noise_cov
         %
         randn('state',1);
         noise_var = sqrt(noise_cov)*randn(size(noise_cov,1),1);
      else;
         noise_var = zeros(size(noise_cov,1),1);
      end;
      %
      w_est = (R_mat*alpha_mat)+noise_var;
      w_cov = (R_partial*Sigma_alpha*R_partial')+noise_cov;
      W_test = w_est'*inverse(w_cov)*w_est;
      if sscorrect==0;
         tests(i,1) = W_test;
         df(i,1) = size(R_mat,1);
         pvals(i,1) = 1-chis_cdf(W_test,df(i,1));
      else;
         df(i,:) = [size(R_mat,1) t+size(R_mat,1)];
         tests(i,1) = (df(i,2)/(T*df(i,1)))*W_test;
         pvals(i,1) = 1-fdis_cdf(tests(i,1),df(i,1),df(i,2));
      end;
   end;
end;

%
% end of waldmgctest.m
%
